"""
These PREP7 commands are used to read model data into the database,
list out the database, and control the numbering of entities in the
database.

"""


def aflist(self, **kwargs):
    """Lists the current data in the database.

    APDL Command: AFLIST

    Notes
    -----
    Lists the current data and specifications in the database.  If batch,
    lists all appropriate data.  If interactive, lists only summaries.
    """
    command = "AFLIST,"
    return self.run(command, **kwargs)


def cdread(self, option="", fname="", ext="", fnamei="", exti="", **kwargs):
    """Reads a file of solid model and database information into the database.

    APDL Command: CDREAD

    Parameters
    ----------
    option
        Selects which data to read:

        ALL - Read all geometry, material property, load, and
        component data (default).  Solid model geometry and loads
        will be read from the file Fnamei.Exti.  All other data
        will be read from the file Fname.Ext.

        DB - Read all database information contained in file
        Fname.Ext. This file should contain all information
        mentioned above except the solid model loads. If reading a
        .CDB file written with the GEOM option of the CDWRITE
        command, element types [ET] compatible with the
        connectivity of the elements on the file must be defined
        prior to reading.

        SOLID - Read the solid model geometry and solid model
        loads from the file Fnamei.Exti.  This file could have
        been written by the CDWRITE or IGESOUT command.

        COMB - Read the combined solid model and database
        information from the file Fname.Ext.

    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    fnamei
        Name of the IGES file and its directory path (248 characters
        maximum, including directory). If you do not specify a directory
        path, it will default to your working directory and you can use all
        248 characters for the file name.

    exti
        Filename extension (eight-character maximum).

    Notes
    -----
    This command causes coded files of solid model (in IGES format) and
    database (in command format) information to be read.  These files are
    normally written by the CDWRITE or IGESOUT command.  Note that the
    active coordinate system in these files has been reset to Cartesian
    (CSYS,0).

    If a set of data exists prior to the CDREAD operation, that data set is
    offset upward to allow the new data to fit without overlap. The
    NOOFFSET command allows this offset to be ignored on a set-by-set
    basis, causing the existing data set to be overwritten with the new
    data set.

    When you write the geometry data using the CDWRITE,GEOM option, you use
    the CDREAD,DB option to read the geometry information.

    Using the CDREAD,COMB option will not write NUMOFF commands to offset
    entity ID numbers if there is no solid model in the database.

    Multiple CDB file imports cannot have elements with real constants in
    one file and section definitions in another. The section attributes
    will override the real constant attributes.  If you use CDREAD to
    import multiple CDB files, define all of the elements using only real
    constants, or using only section definitions.  Combining real constants
    and section definitions is not recommended.

    This command is valid in any processor.
    """
    return self.run(f"CDREAD,{option},{fname},{ext},,{fnamei},{exti}", **kwargs)


def cdwrite(self, option="", fname="", ext="", fnamei="", exti="",
            fmat="", **kwargs):
    """Writes geometry and load database items to a file.

    APDL Command: CDWRITE

    Parameters
    ----------
    option
        Selects which data to write:

        ALL - Write all appropriate geometry, material property,
              load, and component data (default). Two files will
              be produced. Fname.Ext will contain all data items
              mentioned in "Notes", except the solid model
              data. Fnamei.Exti will contain the solid model
              geometry and solid model loads data in the form of
              IGES commands. This option is not valid when
              CDOPT,ANF is active.

        COMB - Write all data mentioned, but to a single file,
               Fname.Ext. Solid model geometry data will be
               written in either IGES or ANF format as specified
               in the CDOPT command, followed by the remainder of
               the data in the form of ANSYS commands. More
               information on these (IGES/ANF) file formats is
               provided in "Notes".

        DB - Write all database information except the solid model
             and solid model loads to Fname.Ext in the form of
             ANSYS commands. This option is not valid when
             CDOPT,ANF is active.

        SOLID - Write only the solid model geometry and solid
                model load data. This output will be in IGES or
                ANF format, as specified in the CDOPT
                command. More information on these (IGES/ANF) file
                formats is provided in "Notes".

        GEOM - Write only element and nodal geometry data. Neither
               solid model geometry nor element attribute data
               will be written. One file, Fname.Ext, will be
               produced. Use CDREAD,DB to read in a file written
               with this option. Element types [ET] compatible
               with the connectivity of the elements on the file
               must first be defined before reading the file in
               with CDREAD,DB.

        CM - Write only node and element component and geometry
        data to Fname.Ext.

        MAT - Write only material property data (both linear and
        nonlinear) to Fname.Ext.

        LOAD - Write only loads for current load step to
        Fname.Ext.

        SECT - Write only section data to Fname.Ext. Pretension
        sections are not included.

    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ext
        Filename extension (eight-character maximum).

    fnamei
        Name of the IGES file and its directory path (248
        characters maximum, including directory). If you do not
        specify a directory path, it will default to your working
        directory and you can use all 248 characters for the file
        name.

    exti
        Filename extension (eight-character maximum).

    fmat
        Format of the output file (defaults to BLOCKED).

        BLOCKED - Blocked format. This format allows faster
                  reading of the output file. The time savings is
                  most significant when BLOCKED is used to read
                  .cdb files associated with very large models.

        UNBLOCKED - Unblocked format.

    Returns
    -------
    str
        Mapdl command output.

    Examples
    --------
    Create a basic block and save it to disk.

    >>> mapdl.prep7()
    >>> mapdl.block(0, 1, 0, 1, 0, 1)
    >>> mapdl.et(1, 186)
    >>> mapdl.esize(0.25)
    >>> mapdl.vmesh('ALL')
    >>> mapdl.cdwrite('DB', '/tmp/mesh.cdb')
     TITLE =
     NUMBER OF ELEMENT TYPES =      1
            64 ELEMENTS CURRENTLY SELECTED.  MAX ELEMENT NUMBER =   64
           425 NODES CURRENTLY SELECTED.     MAX NODE NUMBER =     425
             8 KEYPOINTS CURRENTLY SELECTED. MAX KEYPOINT NUMBER =   8
            12 LINES CURRENTLY SELECTED.     MAX LINE NUMBER =      12
             6 AREAS CURRENTLY SELECTED.     MAX AREA NUMBER =       6
             1 VOLUMES CURRENTLY SELECTED.   MAX VOL. NUMBER =       1
     WRITE ANSYS DATABASE AS AN ANSYS INPUT FILE: /tmp/mesh.cdb

    Optionally load the mesh into Python using the archive reader.

    >>> from ansys.mapdl import reader as pymapdl_reader
    >>> mesh = pymapdl_reader.Archive('/tmp/mesh.cdb')
    >>> mesh
    ANSYS Archive File mesh.cdb
     Number of Nodes:              425
     Number of Elements:           64
     Number of Element Types:      1
     Number of Node Components:    0
     Number of Element Components: 0

    Notes
    -----
    Load data includes the current load step only. Loads applied
    to the solid model (if any) are automatically transferred to
    the finite element model when this command is issued. CDWRITE
    writes out solid model loads for meshed models only. If the
    model is not meshed, the solid model loads cannot be
    saved. Component data include component definitions, but not
    assembly definitions. Appropriate NUMOFF commands are included
    at the beginning of the file; this is to avoid overlap of an
    existing database when the file is read in.

    Element order information (resulting from a WAVES command) is
    not written. The data in the database remain untouched.

    Solution control commands are typically not written to the
    file unless you specifically change a default solution
    setting.

    CDWRITE does not support the GSBDATA and GSGDATA commands, and
    these commands are not written to the file.

    The data may be reread (on a different machine, for example)
    with the CDREAD command. Caution: When the file is read in,
    the NUMOFF,MAT command may cause a mismatch between material
    definitions and material numbers referenced by certain loads
    and element real constants. See NUMOFF for details. Also, be
    aware that the files created by the CDWRITE command explicitly
    set the active coordinate system to Cartesian (CSYS,0).

    You should generally use the blocked format (Fmat = BLOCKED)
    when writing out model data with CDWRITE. This is a compressed
    data format that greatly reduces the time required to read
    large models through the CDREAD command. The blocked and
    unblocked formats are described in Chapter 3 of the Guide to
    Interfacing with ANSYS.

    If you use CDWRITE in any of the derived products (ANSYS Emag,
    ANSYS Professional), then before reading the file, you must
    edit the Jobname.cdb file to remove commands that are not
    available in the respective component product.

    The CDWRITE command writes PART information for any ANSYS
    LS-DYNA input file to the Jobname.cdb file via the EDPREAD
    command. (EDPREAD is not a documented command; it is written
    only when the CDWRITE command is issued.) The PART information
    can be automatically read in via the CDREAD command; however,
    if more than one Jobname.cdb file is read, the PART list from
    the last Jobname.cdb file overwrites the existing PART list of
    the total model. This behavior affects all PART-related
    commands contained in the Jobname.cdb file. You can join
    models, but not PART-related inputs, which you must modify
    using the newly-created PART numbers. In limited cases, an
    update of the PART list (EDWRITE,PUPDATE) is possible; doing
    so requires that no used combination of MAT/TYPE/REAL appears
    more than once in the list.

    The CDWRITE command does not support (for beam meshing) any
    line operation that relies on solid model associativity. For
    example, meshing the areas adjacent to the meshed line,
    plotting the line that contains the orientation nodes, or
    clearing the mesh from the line that contains orientation
    nodes may not work as expected. For more information about
    beam meshing, see Meshing Your Solid Model in the Modeling and
    Meshing Guide.

    IGES and ANF File Formats for Solid Model Geometry Information

    The format used for solid model geometry information is
    determined by the current CDOPT command setting. The default
    format is IGES.

    IGES option (default) to write solid model information (CDOPT,
    IGS):

    Before writing solid model entities, select all corresponding
    lower level entities (ALLSEL,BELOW,ALL).
    """
    command = f"CDWRITE,{option},'{fname}',{ext},,{fnamei},{exti},{fmat}"
    return self.run(command, **kwargs)


def cdopt(self, option="", **kwargs):
    """Specifies format to be used for archiving geometry.

    APDL Command: CDOPT

    Parameters
    ----------
    option
        IGES

        IGES - Write solid model geometry information using IGES format (default).

        ANF - Write solid model geometry information using ANSYS Neutral File format.

        STAT - Print out the current format setting.

    Notes
    -----
    This command controls your solid model geometry format for CDWRITE
    operations. The ANF option affects only the COMB and SOLID options of
    the CDWRITE command. All other options remain unaffected.

    This option setting is saved in the database.
    """
    command = "CDOPT,%s" % (str(option))
    return self.run(command, **kwargs)


def cecheck(self, itemlab="", tolerance="", dof="", **kwargs):
    """Check constraint equations and couplings for rigid body motions.

    APDL Command: CECHECK

    Parameters
    ----------
    itemlab
        Item indicating what is to be checked:

        CE - Check constraint equations only

        CP - Check couplings only

        ALL - Check both CE and CP

    tolerance
        Allowed amount of out-of-balance for any constraint equation or
        coupled set. The default value of 1.0e-6 is usually good.

    dof
        Specifies which DOF is to be checked. Default is RIGID, the usual
        option. Other choices are individual DOF such as UX, ROTZ, etc. or
        THERM. The THERM option will check the constraint equations or
        coupled sets for free thermal expansions, whereas the individual
        DOFs check under rigid body motions. ALL is RIGID and THERM.

    Notes
    -----
    This command imposes a rigid body motion on the nodes attached to the
    constraint equation or coupled set and makes sure that no internal
    forces are generated for such rigid body motions. Generation of
    internal forces by rigid body motions usually indicates an error in the
    equation specification (possibly due to nodal coordinate rotations).
    The THERM option does a similar check to see that no internal forces
    are created by the equations if the body does a free thermal expansion
    (this check assumes a single isotropic coefficient of expansion).
    """
    command = "CECHECK,%s,%s,%s" % (str(itemlab), str(tolerance), str(dof))
    return self.run(command, **kwargs)


def check(self, sele="", levl="", **kwargs):
    """Checks current database items for completeness.

    APDL Command: CHECK

    Parameters
    ----------
    sele
        Specifies which elements are to be checked:

        (blank) - Check all data.

        ESEL - Check only elements in the selected set and unselect any elements not producing
               geometry check messages.  The remaining elements (those
               producing check messages) can then be displayed and
               corrected.  A null set results if no elements produce a
               message.  Issue ESEL,ALL to select all elements before
               proceeding.

    levl
        Used only with Sele = ESEL:

        WARN - Select elements producing warning and error messages.

        ERR - Select only elements producing error messages (default).

    Notes
    -----
    This command will not work if SHPP,OFF has been set. A similar,
    automatic check of all data is done before the solution begins.

    If the "Check Elements" option is invoked through the GUI (menu path
    Main Menu> Preprocessor> Meshing> Check Elems), the CHECK,ESEL logic is
    used to highlight elements in the following way:  good elements are
    blue, elements having warnings are yellow, and bad (error) elements are
    red.

    Note:: : The currently selected set of elements is not changed by this
    GUI function.

    This command is also valid in PREP7.
    """
    command = "CHECK,%s,%s" % (str(sele), str(levl))
    return self.run(command, **kwargs)


def cncheck(self, option="", rid1="", rid2="", rinc="", intertype="",
            trlevel="", cgap="", cpen="", ioff="", **kwargs):
    """Provides and/or adjusts the initial status of contact pairs.

    APDL Command: CNCHECK

    Parameters
    ----------
    option
        Option to be performed:

        DETAIL - List all contact pair properties (default).

        SUMMARY - List only the open/closed status for each contact pair.

        POST - Execute a partial solution to write the initial
               contact configuration to the Jobname.RCN file.

        ADJUST - Physically move contact nodes to the target in
                 order to close a gap or reduce penetration. The
                 initial adjustment is converted to structural
                 displacement values (UX, UY, UZ) and stored in
                 the Jobname.RCN file.

        RESET - Reset target element and contact element key
                options and real constants to their default
                values. This option is not valid for general
                contact.

        AUTO - Automatically sets certain real constants and key
               options to recommended values or settings in order
               to achieve better convergence based on overall
               contact pair behaviors. This option is not valid
               for general contact.

        TRIM - Trim contact pair (remove certain contact and target elements).

        UNSE - Unselect certain contact and target elements.

    rid1, rid2, rinc
        For pair-based contact, the range of real constant pair
        ID's for which Option will be performed. If RID2 is not
        specified, it defaults to RID1. If no value is specified,
        all contact pairs in the selected set of elements are
        considered.

    intertype
        The type of contact interface (pair-based versus general
        contact) to be considered; or the type of contact pair to
        be trimmed/unselected/auto-set.

        (blank) - Include all contact definitions (pair-based and general contact).

        GCN - Include general contact definitions only (not valid
              when Option = RESET or AUTO).

    trlevel
        Trimming level (used only when Option = TRIM or UNSE):

        (blank) - Normal trimming (default): remove/unselect
                  contact and target elements which are in
                  far-field.

        AGGRE - Aggressive trimming: remove/unselect contact and
                target elements which are in far-field, and
                certain elements in near-field.

    Notes
    -----
    The CNCHECK command provides information for
    surface-to-surface, node- to-surface, and line-to-line contact
    pairs (element types TARGE169, TARGE170, CONTA171, CONTA172,
    CONTA173, CONTA174, CONTA175, CONTA176, CONTA177). All contact
    and target elements of interest, along with the solid elements
    and nodes attached to them, must be selected for the command
    to function properly. For performance reasons, the program
    uses a subset of nodes and elements based on the specified
    contact regions (RID1, RID2, RINC) when executing the CNCHECK
    command.

    CNCHECK is available in both the PREP7 and SOLUTION
    processors, but only before the first solve operation (that
    is, only before the first load step or the first substep).

    If the contact and target elements were generated through mesh
    commands (AMESH, LMESH, etc.) instead of the ESURF command,
    you must issue MODMSH,DETACH before CNCHECK. Otherwise,
    CNCHECK will not work correctly.

    The following additional notes are available:

    The command CNCHECK,POST solves the initial contact configuration in
    one substep. After issuing this command, you can postprocess the
    contact result items as you would for any other converged load step;
    however, only the contact status, contact penetration or gap, and
    contact pressure will have meaningful values. Other contact quantities
    (friction stress, sliding distance, chattering) will be available but
    are not useful.

    Because Option = POST forces a solve operation, the PrepPost (PP)
    license does not work with CNCHECK,POST.
    """
    command = "CNCHECK,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(option), str(rid1), str(
        rid2), str(rinc), str(intertype), str(trlevel), str(cgap), str(cpen), str(ioff))
    return self.run(command, **kwargs)


def igesout(self, fname="", ext="", att="", **kwargs):
    """Writes solid model data to a file in IGES Version 5.1 format.

    APDL Command: IGESOUT

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    att
        Attribute key:

        0 - Do not write assigned numbers and attributes of the
            solid model entities to the IGES file (default).

        1 - Write assigned numbers and attributes of solid model
            entities (keypoints, lines, areas, volumes) to the
            IGES file.  Attributes include MAT, TYPE, REAL, and
            ESYS specifications as well as associated solid model
            loads and meshing (keypoint element size, number of
            line divisions and spacing ratio) specifications.

    Notes
    -----
    Causes the selected solid model data to be written to a coded
    file in the IGES Version 5.1 format.  Previous data on this
    file, if any, are overwritten.  Keypoints that are not
    attached to any line are written to the output file as IGES
    entity 116 (Point).  Lines that are not attached to any area
    are written to the output file as either IGES Entity 100
    (Circular Arc), 110 (Line), or 126 (Rational B-Spline Curve)
    depending upon whether the ANSYS entity was defined as an arc,
    straight line, or spline.  Areas are written to the output
    file as IGES Entity 144 (Trimmed Parametric Surface).  Volumes
    are written to the output file as IGES entity 186 (Manifold
    Solid B-Rep Object).  Solid model entities to be written must
    have all corresponding lower level entities selected (use
    ALLSEL,BELOW,ALL) before issuing command.  Concatenated lines
    and areas are not written to the IGES file; however, the
    entities that make up these concatenated entities are written.

    Caution:: : Section properties assigned to areas, lines and
    other solid model entities will not be maintained when the
    model is exported using IGESOUT.

    If you issue the IGESOUT command after generating a beam mesh
    with orientation nodes, the orientation keypoints that were
    specified for the line (LATT) are no longer associated with
    the line and are not written out to the IGES file.  The line
    does not recognize that orientation keypoints were ever
    assigned to it, and the orientation keypoints do not "know"
    that they are orientation keypoints.  Thus the IGESOUT command
    does not support (for beam meshing) any line operation that
    relies on solid model associativity.  For example, meshing the
    areas adjacent to the meshed line, plotting the line that
    contains the orientation nodes, or clearing the mesh from the
    line that contains orientation nodes may not work as expected.
    See Meshing Your Solid Model in the Modeling and Meshing Guide
    for more information about beam meshing.
    """
    return self.run(f"IGESOUT,{fname},{ext},,{att}", **kwargs)


def mfimport(self, fnumb="", option="", fname="", ext="", **kwargs):
    """Imports a new field into a current ANSYS Multi-field solver analysis.

    APDL Command: MFIMPORT

    Parameters
    ----------
    fnumb
        Field number specified by the MFELEM command.

    option
        Selects data to read.

        DB - Reads a CDB file. The CDB file name and extension are specified by Fname and
             Ext.

    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    The field to be imported should be written to a CDB file (CDWRITE
    command). This file is read into the database, offsetting all existing
    element type numbers, node numbers, etc. in order to accommodate the
    imported field. (See the NUMOFF command for information on offset
    capabilities.) It then updates all of the previously issued MFxx
    commands to the new element type numbers. A new field is created using
    the specified field number, which must not currently exist. If there
    are no ANSYS Multi-field solver command files written for the existing
    fields in the database, one will be written for each field with the
    default name (see the MFCMMAND command). A MFCMMAND will be issued for
    the imported field as well.

    Repeat the MFIMPORT command to import additional fields.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = "MFIMPORT,%s,%s,%s,%s" % (
        str(fnumb), str(option), str(fname), str(ext))
    return self.run(command, **kwargs)


def nooffset(self, label="", **kwargs):
    """Prevents the CDREAD command from offsetting specified data items

    APDL Command: NOOFFSET

    Parameters
    ----------
    label
        Specifies items not to be offset.

        NODE - Node numbers

        ELEM - Element numbers

        KP - Keypoint numbers

        LINE - Line numbers

        AREA - Area numbers

        VOLU - Volume numbers

        MAT - Material numbers

        TYPE - Element type numbers

        REAL - Real constant numbers

        CSYS - Coordinate system numbers

        SECN - Section numbers

        CP - Coupled set numbers

        CE - Constraint equation numbers

        CLEAR - All items will be offset

        STATUS - Shows which items are specified notto be offset.

    Notes
    -----
     The NOOFFSET command specifies data items not to be offset by a set of
    data read from a CDREAD command.
    """
    command = "NOOFFSET,%s" % (str(label))
    return self.run(command, **kwargs)


def numcmp(self, label="", **kwargs):
    """Compresses the numbering of defined items.

    APDL Command: NUMCMP

    Parameters
    ----------
    label
        Items to be compressed:

        NODE - Node numbers

        ELEM - Element numbers

        KP - Keypoint numbers

        LINE - Line numbers

        AREA - Area numbers

        VOLU - Volume numbers

        MAT - Material numbers

        TYPE - Element type numbers

        REAL - Real constant numbers

        CP - Coupled set numbers

        SECN - Section numbers

        CE - Constraint equation numbers

        ALL - All item numbers

    Notes
    -----
    The NUMCMP command effectively compresses out unused item
    numbers by renumbering all the items, beginning with one and
    continuing throughout the model.  The renumbering order
    follows the initial item numbering order (that is, compression
    lowers the maximum number by "sliding" numbers down to take
    advantage of unused or skipped numbers).  All defined items
    are renumbered, regardless of whether or not they are actually
    used or selected.  Applicable related items are also checked
    for renumbering as described for the merge operation (NUMMRG).

    Compressing material numbers (NUMCMP,ALL or NUMCMP,MAT) does
    not update the material number referenced by either of the
    following:

    A temperature-dependent convection or surface-to-surface
    radiation load (SF, SFE, SFL, SFA)

    Real constants for multi-material elements (such as SOLID65)

    Compression is usually not required unless memory space is limited and
    there are large gaps in the numbering sequence.
    """
    command = "NUMCMP,%s" % (str(label))
    return self.run(command, **kwargs)


def nummrg(self, label="", toler="", gtoler="", action="", switch="",
           **kwargs):
    """Merges coincident or equivalently defined items.

    APDL Command: NUMMRG

    Parameters
    ----------
    label
        Items to be merged:

        NODE - Nodes

        ELEM - Elements

        KP - Keypoints (will also merge lines, areas, and volumes)

        MAT - Materials

        TYPE - Element types

        REAL - Real constants

        CP - Coupled sets

        CE - Constraint equations

        ALL - All items

    toler
        Range of coincidence.  For Label = NODE and KP, defaults to 1.0E-4
        (based on maximum Cartesian coordinate difference between nodes or
        keypoints).  For Label = MAT, REAL, and CE, defaults to 1.0E-7
        (based on difference of the values normalized by the values).  Only
        items within range are merged.  (For keypoints attached to lines,
        further restrictions apply.  See the GTOLER field and Merging Solid
        Model Entities below.)

    gtoler
        Global solid model tolerance -- used only when merging keypoints
        attached to lines.  If specified, GTOLER will override the internal
        relative solid model tolerance.  See Merging Solid Model Entities
        below.

    action
        Specifies whether to merge or select coincident items.

        SELE - Select coincident items but do not merge. Action = SELE is only valid for Label
               = NODE.

        (Blank) - Merge the coincident items (default).

    switch
        Specifies whether the lowest or highest numbered coincident item is
        retained after the merging operation.  This option does not apply
        to keypoints; i.e., for Label = KP, the lowest numbered keypoint is
        retained regardless of the Switch setting.

        LOW - Retain the lowest numbered coincident item after the merging operation
              (default).

        HIGH - Retain the highest numbered coincident item after the merging operation.

    Notes
    -----
    After issuing the command, the area and volume sizes (ASUM and VSUM)
    may give slightly different results. In order to obtain the same
    results as before, use /FACET, /NORMAL, and ASUM / VSUM.

    The merge operation is useful for tying separate, but coincident, parts
    of a model together. If not all items are to be checked for merging,
    use the select commands (NSEL, ESEL, etc.) to select items.  Only
    selected items are included in the merge operation for nodes,
    keypoints, and elements.

    By default, the merge operation retains the lowest numbered coincident
    item.  Higher numbered coincident items are deleted.  Set Switch to
    HIGH to retain the highest numbered coincident item after the merging
    operation.  Applicable related items are also checked for deleted item
    numbers and if found, are replaced with the retained item number.  For
    example, if nodes are merged, element connectivities (except
    superelements), mesh item range associativity, coupled degrees of
    freedom, constraint equations, master degrees of freedom, gap
    conditions, degree of freedom constraints, nodal force loads, nodal
    surface loads, and nodal body force loads are checked.  Merging
    material numbers [NUMMRG,ALL or NUMMRG,MAT] does not update the
    material number referenced:

    By temperature-dependent film coefficients as part of convection load
    or a temperature-dependent emissivity as part of a surface-to-surface
    radiation load [SF, SFE, SFL, SFA]

    By real constants for multi-material elements (such as SOLID65)

    If a unique load is defined among merged nodes, the value is kept and
    applied to the retained node.  If loads are not unique (not
    recommended), only the value on the lowest node (or highest if Switch =
    HIGH) will be kept, except for "force" loads for which the values will
    be summed if they are not defined using tabular boundary conditions.

    Note:: The unused nodes (not recommended) in elements, couplings,
    constraint equations, etc. may become active after the merge operation.

    The Action field provides the option of visualizing the coincident
    items before the merging operation.

    Caution:: When merging entities in a model that has already been
    meshed, the order in which you issue multiple NUMMRG commands is
    significant.  If you want to merge two adjacent meshed regions that
    have coincident nodes and keypoints, always merge nodes [NUMMRG,NODE]
    before merging keypoints [NUMMRG,KP].  Merging keypoints before nodes
    can result in some of the nodes becoming "orphaned"; that is, the nodes
    lose their association with the solid model.  Orphaned nodes can cause
    certain operations (such as boundary condition transfers, surface load
    transfers, and so on) to fail. However, using NUMMRG should be avoided
    if at all possible, as the procedure outlined above may even cause
    meshing failure, especially after multiple merging and meshing
    operations.

    After a NUMMRG,NODE, is issued, some nodes may be attached to more than
    one solid entity. As a result, subsequent attempts to transfer solid
    model loads to the elements may not be successful. Issue NUMMRG,KP to
    correct this problem. Do NOT issue VCLEAR before issuing NUMMRG,KP.

    For NUMMRG,ELEM, elements must be identical in all aspects, including
    the direction of the element coordinate system.

    For certain solid and shell elements (181, 185, 190, etc) ANSYS will
    interpret coincident faces as internal and eliminate them. To prevent
    this from occurring, shrink the entities by a very small factor to
    delineate coincident items (/SHRINK, 0.0001) and no internal nodes,
    lines, areas or elements will be eliminated.

    When working with solid models, you may have better success with the
    gluing operations (AGLUE, LGLUE, VGLUE). Please read the following
    information when attempting to merge solid model entities.

    Gluing Operations vs. Merging Operations

    Adjacent, touching regions can be joined by gluing them (AGLUE, LGLUE,
    VGLUE) or by merging coincident keypoints (NUMMRG,KP, which also causes
    merging of identical lines, areas, and volumes).  In many situations,
    either approach will work just fine. Some factors, however, may lead to
    a preference for one method over the other.

    Geometric Configuration

    Gluing is possible regardless of the initial alignment or offset of the
    input entities. Keypoint merging is  possible only if each keypoint on
    one side of the face to be joined is matched by a coincident keypoint
    on the other side. This is commonly the case after a symmetry
    reflection (ARSYM or VSYMM) or a copy (AGEN or VGEN),  especially for a
    model built entirely in ANSYS rather than imported from a CAD system.
    When the geometry is  extremely precise, and the configuration is
    correct for  keypoint merging, NUMMRG is more efficient and robust than
    AGLUE or VGLUE.
    """
    command = "NUMMRG,%s,%s,%s,%s,%s" % (str(label), str(
        toler), str(gtoler), str(action), str(switch))
    return self.run(command, **kwargs)


def numoff(self, label="", value="", **kwargs):
    """Adds a number offset to defined items.

    APDL Command: NUMOFF

    Parameters
    ----------
    label
        Apply offset number to one of the following sets of items:

        NODE - Nodes

        ELEM - Elements

        KP - Keypoints

        LINE - Lines

        AREA - Areas

        VOLU - Volumes

        MAT - Materials

        TYPE - Element types

        REAL - Real constants

        CP - Coupled sets

        SECN - Section numbers

        CE - Constraint equations

        CSYS - Coordinate systems

    value
        Offset number value (cannot be negative).

    Notes
    -----
    Useful for offsetting current model data to prevent overlap if another
    model is read in. CDWRITE automatically writes the appropriate NUMOFF
    commands followed by the model data to File.CDB.  Therefore, when the
    file is read, any model already existing in the database is offset
    before the model data on the file is read.

    Offsetting material numbers with this command  [NUMOFF,MAT] does not
    update the material number referenced by either of the following:

    A temperature-dependent convection or surface-to-surface radiation load
    [SF, SFE, SFL, SFA]

    Real constants for multi-material elements (such as SOLID65).

    Therefore, a mismatch may exist between the material definitions and
    the material numbers referenced.
    """
    command = "NUMOFF,%s,%s" % (str(label), str(value))
    return self.run(command, **kwargs)


def numstr(self, label="", value="", **kwargs):
    """Establishes starting numbers for automatically numbered items.

    APDL Command: NUMSTR

    Parameters
    ----------
    label
        Apply starting number to one of the following sets of items:

        NODE - Node numbers.  Value defaults (and is continually reset) to 1 + maximum node
               number in model.  Cannot be reset lower.

        ELEM - Element numbers.  Value defaults (and is continually reset) to 1 + maximum
               element number in model.  Cannot be reset lower.

        KP - Keypoint numbers.  Value defaults to 1.  Only undefined numbers are used.
             Existing keypoints are not overwritten.

        LINE - Line numbers.  Value defaults to 1.  Only undefined numbers are used.  Existing
               lines are not overwritten.

        AREA - Area numbers.  Value defaults to 1.  Only undefined numbers are used.  Existing
               areas are not overwritten.

        VOLU - Volume numbers.  Value defaults to 1.  Only undefined numbers are used.
               Existing volumes are not overwritten.

        DEFA - Default.  Returns all starting numbers to their default values.

    value
        Starting number value.

    Notes
    -----
    Establishes starting numbers for various items that may have numbers
    automatically assigned (such as element numbers with the EGEN command,
    and node and solid model entity numbers with the mesh [AMESH, VMESH,
    etc.] commands).  Use NUMSTR,STAT to display settings.  Use NUMSTR,DEFA
    to reset all specifications back to defaults.  Defaults may be lowered
    by deleting and compressing items (i.e., NDELE and NUMCMP,NODE for
    nodes, etc.).

    Note:: : A mesh clear operation (VCLEAR, ACLEAR, LCLEAR, and KCLEAR)
    automatically sets starting node and element numbers to the highest
    unused numbers.  If a specific starting node or element number is
    desired, issue NUMSTR after the clear operation.
    """
    command = "NUMSTR,%s,%s" % (str(label), str(value))
    return self.run(command, **kwargs)



def iclwid(self, factor="", **kwargs):
    """Scales the line width of circuit builder icons.

    APDL Command: /ICLWID

    Parameters
    ----------
    factor
        Multiplication factor applied to the default line width (defaults
        to 1). The minimum is 1 and the maximum is 6.

    Notes
    -----
    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"/ICLWID,{factor}"
    return self.run(command, **kwargs)


def icscale(self, wn="", factor="", **kwargs):
    """Scales the icon size for elements supported in the circuit builder.

    APDL Command: /ICSCALE

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies (defaults to 1).

    factor
        Factor applied to the default icon size (defaults to 1).

    Notes
    -----
    Scaling the icon size can provide better visualization of the circuit
    components when using the Circuit Builder (an interactive builder
    available in the ANSYS GUI).

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"/ICSCALE,{wn},{factor}"
    return self.run(command, **kwargs)


def ratio(self, wn="", ratox="", ratoy="", **kwargs):
    """Distorts the object geometry.

    APDL Command: /RATIO

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies (defaults to 1).

    ratox
        Distort object in the window X direction by this factor (defaults
        to 1.0).

    ratoy
        Distort object in the window Y direction by this factor (defaults
        to 1.0).

    Notes
    -----
    Distorts the object geometry in a particular direction.  An example of
    this command's use would be to allow long narrow sections to be
    distorted to a more square area for better display visualization.

    This command is valid in any processor.
    """
    command = f"/RATIO,{wn},{ratox},{ratoy}"
    return self.run(command, **kwargs)


def shrink(self, ratio="", **kwargs):
    """Shrinks elements, lines, areas, and volumes for display clarity.

    APDL Command: /SHRINK

    Parameters
    ----------
    ratio
        Shrinkage ratio (input as a decimal (0.0 to 0.5)).  Defaults to 0.0
        (no shrinkage).  Values greater than 0.5 default to 0.1 (10%
        shrinkage).

    Notes
    -----
    Shrinks the elements, lines, areas, and volumes so that adjacent
    entities are separated for clarity.  Portions of this command are not
    supported by PowerGraphics [/GRAPHICS,POWER].

    If only the common lines of non-coplanar faces are drawn (as per the
    /EDGE command), then this command is ignored.

    This command is valid in any processor.
    """
    command = f"/SHRINK,{ratio}"
    return self.run(command, **kwargs)


def sscale(self, wn="", smult="", **kwargs):
    """Sets the contour multiplier for topographic displays.

    APDL Command: /SSCALE

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies (defaults to 1).

    smult
        Contour multiplier that factors in results based on the product of
        the multiplier and the result being plotted. Defaults to 0.0 (no
        topographic effects).

    Notes
    -----
    Use this command to scale values to the geometry when the contours are
    shown elevated.  For section displays [/TYPE], the elevation is
    performed perpendicular to the section face.

    Nonzero contour multipliers factoring in large results (stresses or
    displacements) can produce very large distortion, causing images to
    disappear.  To bring a distorted image back into view, reduce the
    contour multiplier value.

    Portions of this command are not supported by PowerGraphics
    [/GRAPHICS,POWER].
    """
    command = f"/SSCALE,{wn},{smult}"
    return self.run(command, **kwargs)


def txtre(self, lab="", num="", n1="", n2="", ninc="", **kwargs):
    """Controls application of texture to selected items.

    APDL Command: /TXTRE

    Parameters
    ----------
    lab
        You can apply texture according to the following labels:

        ELEM - Apply texture to elements N1 through N2 in steps of NINC.

        AREA - Apply texture to areas N1 through N2 in steps of NINC.

        VOLU - Apply texture to volumes N1 through N2 in steps of NINC.

        CM - Apply texture to the component named in N1. N2 and
        NINC are ignored.

        ON, OFF - Sets the specified texture display on or
        off. All other fields are ignored.

        File - If Lab = File, the command format is /TXTRE, File,
               Key_Index, Fname, Fext, --, Format (This variant of
               the command is applicable to 2-D drivers).

        Key_Index - The texture index associated with the file. If
                    the number fifty-one (51) is used, the
                    imported bitmap will be used as the window's
                    logo.

        Fname - File name and directory path (248 characters
                maximum, including the characters needed for the
                directory path).  An unspecified directory path
                defaults to the working directory; in this case,
                you can use all 248 characters for the file name.

        Fext - Filename extension (eight-character maximum).

        Format - The file format. If Format = 0, the file is a
                 pixmap (Linux) or Bitmap (PC).  The file cannot
                 contain a compressed image, and the PC file must
                 be 8 or 24 bit BI_RGB format. If Format = 1 or
                 JPEG, then the file is in JPEG (Joint
                 Photographic Experts Group) format. If Format = 2
                 or PNG, then the file is in PNG (Portable Network
                 Graphics) format.

    num
        Select the texture index number from the following list:

        0 - No Texturing

        1 - Aluminum

        2 -  Aluminum, Brushed

        3 - Steel With Bumps

        4 - Steel, Embossed

        5 - Iron

        6 - Steel, Pattern

        7 - Steel, Riveted

        8 - Steel, Scratched

        9 - Tin

        10 - Metal

        11 - Steel, Etched

        12 - Metal, Hot

        13 - Iron, Grainy

        14 - Metal, Rusty

        15 - Brick

        16 - Block

        17 - Wood

        18 - Wood, Light

        19 - Wood, Walnut

        20 - Plastic, Hard Blue

        21 - Plastic, Light Blue

        22 - Plastic, Hard Red

        31 - Gold

        32 - Brass

        33 - Silver

        34 -  Plastic, Black

        35 - Plastic, Ivory

        36 - Plastic, Blue

        37 - Plastic, Red

        38 - Plastic, Yellow

        39 - Plastic, Green

        40 - Plastic, Brown

    n1, n2, ninc
        Apply texture to Lab items numbered N1 through N2 in steps of NINC
        (defaults to 1). If Lab = CM, then N1 is used to for the component
        name and N2 and NINC are ignored. If Lab = ELEM, AREA, or VOLU and
        N1 = blank or ALL, then the specified texture will be applied to
        all entities of type Lab. If N1 = P, then graphical picking is
        enabled.

    Notes
    -----
     This command is available for 3-D Open GL devices. 2-D devices are
    supported only for the Lab = File variation of the command, allowing
    imported bitmaps to be used for texturing and annotation. Textures can
    affect the speed of many of your display operations. You can increase
    the speed by temporarily turning the textures off (Utility Menu>
    PlotCtrls> Style> Texturing(3D)> Display Texturing). This menu
    selection toggles your textures on and off. When textures are toggled
    off, all of the texture information is retained and reapplied when
    texturing is toggled back on.

    For some displays, the texture will appear distorted because of a
    technique used to enhance 3-D displays (/DV3D,TRIS,1). Disabling this
    function (/DV3D,TRIS,0) will improve the quality of some texture
    displays. Disabling the TRIS option of the /DV3D command will slow down
    3-D displays significantly. Be sure to reapply the TRIS option after
    you obtain a satisfactory output.

    Specifying /TXTRE,DEFA removes all texturing.
    """
    command = f"/TXTRE,{lab},{num},{n1},{n2},{ninc}"
    return self.run(command, **kwargs)


def vscale(self, wn="", vratio="", key="", **kwargs):
    """Scales the length of displayed vectors.

    APDL Command: /VSCALE

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies (defaults to 1).

    vratio
        Ratio value applied to the automatically calculated scale factor
        (defaults to 1.0, i.e., use scale factor as automatically
        calculated).

    key
        Relative scaling key:

        0 - Use relative length scaling among vectors based on magnitudes.

        1 - Use uniform length scaling for all vector lengths.

    Notes
    -----
    Allows scaling of the vector length displayed with the PLVECT command
    of POST1 and the /PBC and /PSF commands.  Also allows the scaling of
    the  element (i.e., /PSYMB,ESYS) and the nodal (i.e., /PSYMB,NDIR)
    coordinate system symbols.

    This command is valid in any processor.
    """
    command = f"/VSCALE,{wn},{vratio},{key}"
    return self.run(command, **kwargs)



def gp(self, node1="", node2="", lab="", stif="", gap="", damp="",
       **kwargs):
    """Defines a gap condition for transient analyses.

    APDL Command: GP

    Parameters
    ----------
    node1
        Node I of gap.  If NODE1 = P, graphical picking is enabled and all
        remaining command fields are ignored (valid only in the GUI).

    node2
        Node J of gap (must be different from NODE1).  Non-grounded gap
        nodes must be defined as master degrees of freedom or be
        unconstrained, active DOF in a full analysis type.  Grounded gap
        nodes (those not defined as MDOF) need not appear elsewhere in the
        model.

    lab
        Direction of gap action in the nodal coordinate system (implied
        from the following force labels): FX, FY, FZ, MX, MY, MZ.

    stif
        Stiffness (Force/Length) of closed gap (may be positive or
        negative).

    gap
        Initial size of gap.  A zero (or positive) value assumes an
        initially open gap.  A negative value defines an interference
        condition.  For a rotational gap, GAP should be in radians.

    damp
        Damping coefficient (Force*Time/Length) of closed gap using pseudo
        velocity (Newmark finite difference expansion scheme).

    Notes
    -----
    Defines a gap condition for the mode superposition transient analysis
    (ANTYPE,TRANS with TRNOPT,MSUP). If used in SOLUTION, this command is
    valid only within the first load step. Gap conditions specified in
    subsequent load steps are ignored.

    Repeat GP command for additional gap conditions. Gaps are numbered
    sequentially as input.

    Note:: : Gaps may be renumbered by the program during the solution (see
    output listing)

    The mode-superposition transient analysis does not allow gap action
    with the standard gap elements. However, you can define gap conditions
    which are similar to gap elements; gap conditions can be specified
    between surfaces that are expected to contact (impact) each other
    during the transient. The gap condition simulates the basic gap action
    of the COMBIN40 element.

    The gap condition is treated as an explicit force (equal to the
    interference times contact stiffness) and affects only the load vector
    calculation and not the stiffness matrix. The interference is
    calculated from the displacement extrapolated from the previous time
    points.

    Gap conditions can only be defined between two master degree of freedom
    (DOF) nodes or between master DOF nodes and ground, as shown in the
    following figure.

    Master degrees of freedom are the unconstrained and active degrees of
    freedom. Gap nodes not defined as active degrees of freedom or attached
    to an element are assumed to be grounded. Grounded gap nodes do not
    need a spatial location, nor do they need to be located on an element.

    Gap conditions may be defined in parallel (across the same nodes), with
    varying gap and stiffness values, to simulate a nonlinear (piecewise)
    force-deflection curve.

    The gap direction is determined from the force label input on the GP
    command; i.e., FX defines a translational gap acting in the UX nodal
    degree of freedom direction, and MZ defines a rotational gap acting in
    the nodal ROTZ degree of freedom direction. The actual degree of
    freedom directions available for a particular node depends upon the
    degrees of freedom associated with the element types [ET] at that node.

    If the coordinate systems of the nodes connecting the gap are rotated
    relative to each other, the same degree of freedom may be in different
    directions. The gap, however, assumes only a one-dimensional action.
    Nodes I and J may be anywhere in space (preferably coincident).  No
    moment effects are included due to noncoincident nodes. That is, if the
    nodes are offset from the line of action, moment equilibrium may not be
    satisfied.

    The contact stiffness value represents the stiffness of the closed gap.
    Stiffness values are related to the integration time step size and
    should be physically reasonable. High stiffness will require a small
    integration time step; otherwise, due to the displacement
    extrapolation, the solution may go unstable. Negative stiffness values
    may be used with gaps in parallel to produce a decreasing force-
    deflection curve.

    The order of specifying the gap nodes is important; i.e., a gap
    condition connecting two nodes will act differently depending upon
    which node is specified first on the GP command.  For example, for Node
    1 at X = 0.0, Node 2 at X = 0.1, and the gap defined from Node 1 to 2,
    a displacement of Node 1 greater than Node 2 will cause the gap to
    close.  For the gap defined from Node 2 to 1, a displacement of Node 2
    greater than Node 1 will cause the gap to close (like a hook action).
    In general, the gap closes whenever the separation (defined as UJ - UI
    + GAP) is negative.  UJ is the displacement of node J, UI is the
    displacement of node I, and GAP is the input gap value.  The gap force
    output appears in the printout only for the time steps for which the
    gap is closed.  A negative spring force is always associated with a
    closed gap (even with the hook option).

    Some guidelines to define gap conditions are presented below:

    Use enough gap conditions to obtain a smooth contact stress
    distribution between the contacting surfaces.

    Define a reasonable gap stiffness. If the stiffness is too low, the
    contacting surfaces may overlap too much. If the stiffness is too high,
    a very small time step will be required during impact. A general
    recommendation is to specify a gap stiffness that is one or two orders
    of magnitude higher than the adjacent element stiffness. You can
    estimate the adjacent element stiffness using AE/L, where A is the
    contributing area around the gap condition, E is the elastic modulus of
    the softer material at the interface, and L is the depth of the first
    layer of elements at the interface.

    A mode-superposition transient using the nonlinear gap damping provided
    through the DAMP field runs faster than a full transient analysis using
    a gap element (COMBIN40).

    Use the GPLIST command to list gap conditions and the GPDELE command to
    delete gap conditions.

    This command is also valid in PREP7.
    """
    command = f"GP,{node1},{node2},{lab},{stif},{gap},{damp}"
    return self.run(command, **kwargs)


def gpdele(self, gap1="", gap2="", ginc="", **kwargs):
    """Deletes gap conditions.

    APDL Command: GPDELE

    Parameters
    ----------
    gap1, gap2, ginc
        Delete gap conditions from GAP1 to GAP2 (defaults to GAP1) in steps
        of GINC  (defaults to 1).

    Notes
    -----
    Deletes gap conditions defined with the GP command.  Gap conditions
    following those deleted are automatically compressed and renumbered.
    If used in SOLUTION, this command is valid only within the first load
    step.

    This command is also valid in PREP7.
    """
    command = f"GPDELE,{gap1},{gap2},{ginc}"
    return self.run(command, **kwargs)


def gplist(self, gap1="", gap2="", ginc="", **kwargs):
    """Lists the gap conditions.

    APDL Command: GPLIST

    Parameters
    ----------
    gap1, gap2, ginc
        List gap conditions from GAP1 to GAP2 (GAP2 defaults to GAP1) in
        steps of GINC  (defaults to 1).  If GAP1 = ALL (default), GAP2 and
        GINC  are ignored and all gap conditions are listed.

    Notes
    -----
    This command is valid in any processor.
    """
    command = f"GPLIST,{gap1},{gap2},{ginc}"
    return self.run(command, **kwargs)



def cfact(self, rfacta="", ifacta="", rfactb="", ifactb="", rfactc="",
          ifactc="", **kwargs):
    """Defines complex scaling factors to be used with operations.

    APDL Command: CFACT

    Parameters
    ----------
    rfacta
        Real portion of the complex scale factor used in place of FACTA.

    ifacta
        Imaginary portion of the complex scale factor used in place of
        FACTA.

    rfactb
        Real portion of the complex scale factor used in place of FACTB.

    ifactb
        Imaginary portion of the complex scale factor used in place of
        FACTB.

    rfactc
        Real portion of the complex scale factor used in place of FACTC.

    ifactc
        Imaginary portion of the complex scale factor used in place of
        FACTC.

    Notes
    -----
    Defines complex scale factors to be used with the operations [ADD,
    PROD, etc.].  If this command is supplied, these complex factors
    override any real factors (FACTA, FACTB, FACTC) supplied on the
    operation commands.  Factors are typically involved in scaling a
    specified variable, such as in the term FACTA x IA of the ADD command
    to scale variable IA before the ADD operation.

    When the CFACT command is active, defaults are as follows: 1) if the
    complex factor is not specified, but the variable upon which it acts
    (such as IA) is specified, the factor defaults to 1.0+i0.0;  2) if the
    variable upon which the factor operates is not specified, but the
    factor is specified, the variable defaults to 1.0 so that the term in
    the operation becomes the complex factor itself;  3) if neither the
    factor nor the variable number is supplied, the term is omitted from
    the operation.  Once the operation (such as the ADD command) has been
    processed, the CFACT command becomes inactive and must be specified
    again if it is to be used.
    """
    command = f"CFACT,{rfacta},{ifacta},{rfactb},{ifactb},{rfactc},{ifactc}"
    return self.run(command, **kwargs)


def force(self, lab="", **kwargs):
    """Selects the element nodal force type for output.

    APDL Command: FORCE

    Parameters
    ----------
    lab
        Type of force to be associated with the force items:

        TOTAL - Total forces (static, damping, and inertia).

        STATIC - Static forces.

        DAMP - Damping forces.

        INERT - Inertia forces.

    Notes
    -----
    FORCE selects the element nodal force type for output with the POST1
    PRESOL, PLESOL, PRRFOR, NFORCE, FSUM, etc. commands, the POST26 ESOL
    command, and reaction force plotting [/PBC].  For example, FORCE,STATIC
    causes item F of the PRESOL command to be the static forces for the
    elements processed. Element member forces (such as those available for
    beams and shells and processed by Item and Sequence number) are not
    affected by this command. The SMISC records extract the static force.

    The PRRSOL command is not valid with FORCE.  Use the PRRFOR command,
    which provides the same functionality as PRRSOL, instead.

    Use the FORCE command prior to any load case operations (LCOPER) to
    insure the correct element nodal force combinations.

    In POST26, the ESOL data stored is based on the active FORCE
    specification at the time the data is stored. To store data at various
    specifications (for example, static and inertia forces), issue a STORE
    command before each new specification.

    The FORCE command cannot be used to extract static, damping, and
    inertial forces for MPC184 joint elements.

    To retrieve the different force types, use the *GET command with
    Entity=ELEM and Item1=EFOR.

    The FORCE command is not supported in a spectrum analysis. You can
    specify the force type directly on the combination method commands
    (ForceType on the PSDCOM, SRSS, CQC, etc. commands).

    The FORCE command is not supported in a modal analysis.
    """
    command = f"FORCE,{lab}"
    return self.run(command, **kwargs)


def layerp26(self, num="", **kwargs):
    """Specifies the element layer for which data are to be stored.

    APDL Command: LAYERP26

    Parameters
    ----------
    num
        Layer-processing mode:

        N - The layer number to process. The default value is 1.

    Notes
    -----
    Defines the element layer for which results data are to be stored for
    postprocessing.  Applies to stress and strain data for layered elements
    BEAM161, SHELL163, SHELL181, SOLID185, SOLID186, SOLSH190, SHELL208,
    SHELL209, SHELL281, REINF265, and ELBOW290.

    The SHELL command can be used (for shell elements) to specify a
    location (TOP, MID, BOT) within the layer for selection on the ESOL
    command. Transverse shear stresses for MID are linearly averaged from
    TOP and BOT, and do not reflect a parabolic distribution. Setting
    KEYOPT(8) = 2 for SHELL181, SHELL208, SHELL209, SHELL281, and ELBOW290
    writes the mid-surface values directly to the results file and yields
    more accurate values than linear averaging.

    That this command cannot be used for energy output, as energy is a per-
    element quantity.

    When using the LAYERP26 command with SHELL181, SOLID185, SOLID186,
    SOLSH190, SHELL208, or SHELL209, KEYOPT(8) must be set to 1 (or 2 for
    SHELL181, SHELL208, SHELL209, SHELL281, and ELBOW290) in order to store
    results for all layers.

    For the ANSYS LS-DYNA product, this command works differently than
    described above.  For SHELL163 and BEAM161, you must first use EDINT
    during the solution phase to define the integration points for which
    you want output data.  Be aware that the output location for SHELL163
    data is always at the integration point, so "top" and "bottom" refer to
    the top or bottom integration point, not necessarily the top or bottom
    surface.  For more information, see the ANSYS LS-DYNA User's Guide.

    In POST26, the ESOL data stored is based on the active LAYERP26
    specification at the time the data is stored. To store data at various
    specifications (for example, layers 2 and 5), issue a STORE command
    before each new specification.
    """
    command = f"LAYERP26,{num}"
    return self.run(command, **kwargs)


def shell(self, loc="", **kwargs):
    """Selects a shell element or shell layer location for results output.

    APDL Command: SHELL

    Parameters
    ----------
    loc
        Location within shell element (or layer) to obtain stress results:

        TOP - Top of shell element (or layer) (default).

        MID - Middle of shell element (or layer). The default method averages the TOP and BOT
              values to obtain a mid value. Setting KEYOPT(8) = 2 for
              SHELL181, SHELL208, SHELL209, and ELBOW290 uses MID results
              obtained directly from the results file.

        BOT - Bottom of shell element (or layer).

    Notes
    -----
    Selects the location within a shell element (or a shell layer) for
    results output (nodal stresses, strains, etc.).  Applies to POST1
    selects, sorts, and output [NSEL, NSORT, PRNSOL, PLNSOL, PRPATH,
    PLPATH, etc.], and is used for storage with the POST26 ESOL command.
    For example, SHELL,TOP causes item S of the POST1 PRNSOL command or the
    POST26 ESOL command to be the stresses at the top of the shell
    elements.  For layered shell elements, use the LAYER (POST1) or
    LAYERP26 (POST26) command to select the layer. The SHELL command does
    not apply to the layered thermal shell elements, SHELL131 and SHELL132.

    For PowerGraphics [/GRAPHICS,POWER], the SHELL,MID command affects both
    the printed output and the displayed results, while the SHELL (TOP or
    BOT) command prints and displays both the top and bottom layers
    simultaneously. Note that /CYCEXPAND,ON automatically turns on
    PowerGraphics; however, for cyclic mode-superposition harmonic
    postprocessing (CYCFILES), the SHELL command prints and displays only
    the requested layer.

    In POST26, the ESOL data stored is based on the active SHELL
    specification at the time the data is stored. To store data at various
    specifications (for example, stresses at the top and bottom locations),
    issue a STORE command before each new specification.
    """
    command = f"SHELL,{loc}"
    return self.run(command, **kwargs)


def tvar(self, key="", **kwargs):
    """Changes time to the cumulative iteration number.

    APDL Command: TVAR

    Parameters
    ----------
    key
        Time key:

        0 - Time is used for the variable TIME.

        1 - NCUMIT is used for the variable TIME.

    Notes
    -----
    Changes the meaning of the time variable to the cumulative iteration
    number (NCUMIT) variable.  Data can be read from the file, printed, and
    displayed as a function of NCUMIT rather than time.  All POST26
    descriptions applying to TIME then apply to NCUMIT.
    """
    command = f"TVAR,{key}"
    return self.run(command, **kwargs)



from ._commands import (hidden, session, database, preproc, aux15_, map_cmd,
                        aux2_, aux3_, aux12_, reduced, apdl, post26_, solution,
                        post1_, graphics_, display_, conn)

class Commands:
    """Wrapped MAPDL commands"""

    ##################
    # Session commands
    ##################

    config = session.run_controls.config
    cwd = session.run_controls.cwd
    filname = session.run_controls.filname
    keyw = session.run_controls.keyw
    memm = session.run_controls.memm
    nerr = session.run_controls.nerr
    pause = session.run_controls.pause
    slashstatus = session.run_controls.slashstatus
    starstatus = session.run_controls.starstatus
    syp = session.run_controls.syp
    sys = session.run_controls.sys
    unpause = session.run_controls.unpause

    anstoaqwa = session.files.anstoaqwa
    anstoasas = session.files.anstoasas
    assign = session.files.assign
    copy = session.files.copy
    slashdelete = session.files.slashdelete
    fcomp = session.files.fcomp
    lgwrite = session.files.lgwrite
    starlist = session.files.starlist
    rename = session.files.rename
    slashclog = session.files.slashclog
    slashfdele = session.files.slashfdele

    com = session.list_controls.com
    golist = session.list_controls.golist
    gopr = session.list_controls.gopr
    nolist = session.list_controls.nolist
    nopr = session.list_controls.nopr

    aux2 = session.processor_entry.aux2
    aux3 = session.processor_entry.aux3
    aux12 = session.processor_entry.aux12
    aux15 = session.processor_entry.aux15
    finish = session.processor_entry.finish
    map = session.processor_entry.map
    post1 = session.processor_entry.post1
    post26 = session.processor_entry.post26
    prep7 = session.processor_entry.prep7
    quit = session.processor_entry.quit
    slashsolu = session.processor_entry.slashsolu

    ###################
    # Database commands
    ###################
    cm = database.components.cm
    cmdele = database.components.cmdele
    cmedit = database.components.cmedit
    cmgrp = database.components.cmgrp
    cmlist = database.components.cmlist
    cmmod = database.components.cmmod
    cmplot = database.components.cmplot
    cmsel = database.components.cmsel
    cmwrite = database.components.cmwrite

    clocal = database.coord_sys.clocal
    cs = database.coord_sys.cs
    cscir = database.coord_sys.cscir
    csdele = database.coord_sys.csdele
    cskp = database.coord_sys.cskp
    cslist = database.coord_sys.cslist
    cswpla = database.coord_sys.cswpla
    csys = database.coord_sys.csys
    local = database.coord_sys.local

    fitem = database.picking.fitem
    flst = database.picking.flst

    allsel = database.selecting.allsel
    asll = database.selecting.asll
    asel = database.selecting.asel
    aslv = database.selecting.aslv
    dofsel = database.selecting.dofsel
    esel = database.selecting.esel
    esla = database.selecting.esla
    esll = database.selecting.esll
    esln = database.selecting.esln
    eslv = database.selecting.eslv
    ksel = database.selecting.ksel
    ksll = database.selecting.ksll
    ksln = database.selecting.ksln
    lsel = database.selecting.lsel
    lsla = database.selecting.lsla
    lslk = database.selecting.lslk
    nsel = database.selecting.nsel
    nsla = database.selecting.nsla
    nsle = database.selecting.nsle
    nslk = database.selecting.nslk
    nsll = database.selecting.nsll
    nslv = database.selecting.nslv
    partsel = database.selecting.partsel
    vsel = database.selecting.vsel
    vsla = database.selecting.vsla

    resume = database.setup.resume
    save = database.setup.save
    smbc = database.setup.smbc
    stat = database.setup.stat
    stitle = database.setup.stitle
    title = database.setup.title
    units = database.setup.units

    kwpave = database.working_plane.kwpave
    kwplan = database.working_plane.kwplan
    lwplan = database.working_plane.lwplan
    nwpave = database.working_plane.nwpave
    nwplan = database.working_plane.nwplan
    wpave = database.working_plane.wpave
    wpcsys = database.working_plane.wpcsys
    wplane = database.working_plane.wplane
    wpoffs = database.working_plane.wpoffs
    wprota = database.working_plane.wprota
    wpstyl = database.working_plane.wpstyl

    # mapping processor
    ftype = map_cmd.ftype
    map_ = map_cmd.map_
    plgeom = map_cmd.plgeom
    plmap = map_cmd.plmap
    read = map_cmd.read
    slashmap = map_cmd.slashmap
    target = map_cmd.target
    writemap = map_cmd.writemap

    # AUX15
    igesin = aux15_.igesin
    ioptn = aux15_.ioptn

    # AUX2
    combine = aux2_.bin_manip.combine
    hbmat = aux2_.bin_manip.hbmat
    psmat = aux2_.bin_manip.psmat
    dump = aux2_.bin_dump.dump
    fileaux2 = aux2_.bin_dump.fileaux2
    form = aux2_.bin_dump.form
    ptr = aux2_.bin_dump.ptr

    # AUX3
    compress = aux3_.compress
    delete = aux3_.delete
    fileaux3 = aux3_.fileaux3
    list = aux3_.list
    modify = aux3_.modify
    undelete = aux3_.undelete

    # AUX12
    emis = aux12_.radiation_mat.emis
    geom = aux12_.radiation_mat.geom
    mprint = aux12_.radiation_mat.mprint
    space = aux12_.radiation_mat.space
    vtype = aux12_.radiation_mat.vtype
    write = aux12_.radiation_mat.write
    stef = aux12_.general_radiation.stef
    toffst = aux12_.general_radiation.toffst
    hemiopt = aux12_.radiosity_solver.hemiopt
    radopt = aux12_.radiosity_solver.radopt
    spcnod = aux12_.radiosity_solver.spcnod
    spctemp = aux12_.radiosity_solver.spctemp
    v2dopt = aux12_.radiosity_solver.v2dopt
    vfopt = aux12_.radiosity_solver.vfopt
    vfsm = aux12_.radiosity_solver.vfsm
    vfquery = aux12_.radiosity_solver.vfquery

    # reduced processor
    rmndisp = reduced.preparation.rmndisp
    rmnevec = reduced.preparation.rmnevec
    rmresume = reduced.setup.rmresume
    rmsave = reduced.setup.rmsave
    dcvswp = reduced.use_pass.dcvswp
    rmlvscale = reduced.use_pass.rmlvscale
    rmuse = reduced.use_pass.rmuse
    rmalist = reduced.generation.rmalist
    rmanl = reduced.generation.rmanl
    rmaster = reduced.generation.rmaster
    rmcap = reduced.generation.rmcap
    rmclist = reduced.generation.rmclist
    rmmlist = reduced.generation.rmmlist
    rmmrange = reduced.generation.rmmrange
    rmmselect = reduced.generation.rmmselect
    rmporder = reduced.generation.rmporder
    rmrgenerate = reduced.generation.rmrgenerate
    rmroptions = reduced.generation.rmroptions
    rmrplot = reduced.generation.rmrplot
    rmrstatus = reduced.generation.rmrstatus
    rmsmple = reduced.generation.rmsmple
    rmxport = reduced.generation.rmxport

    ################
    # PREP7 commands
    ################
    aflist = preproc.database.aflist
    cdread = preproc.database.cdread
    cdwrite = preproc.database.cdwrite
    cdopt = preproc.database.cdopt
    cecheck = preproc.database.cecheck
    check = preproc.database.check
    cncheck = preproc.database.cncheck
    igesout = preproc.database.igesout
    mfimport = preproc.database.mfimport
    nooffset = preproc.database.nooffset
    numcmp = preproc.database.numcmp
    nummrg = preproc.database.nummrg
    numoff = preproc.database.numoff
    numstr = preproc.database.numstr
    nsvr = preproc.element_type.nsvr
    dof = preproc.element_type.dof
    elbow = preproc.element_type.elbow
    et = preproc.element_type.et
    etchg = preproc.element_type.etchg
    etcontrol = preproc.element_type.etcontrol
    etdele = preproc.element_type.etdele
    etlist = preproc.element_type.etlist
    keyopt = preproc.element_type.keyopt
    r = preproc.real_constants.r
    rdele = preproc.real_constants.rdele
    rlist = preproc.real_constants.rlist
    rmodif = preproc.real_constants.rmodif
    rmore = preproc.real_constants.rmore
    setfgap = preproc.real_constants.setfgap
    emunit = preproc.materials.emunit
    mp = preproc.materials.mp
    mpamod = preproc.materials.mpamod
    mpchg = preproc.materials.mpchg
    mpcopy = preproc.materials.mpcopy
    mpdata = preproc.materials.mpdata
    mpdele = preproc.materials.mpdele
    mpdres = preproc.materials.mpdres
    mplib = preproc.materials.mplib
    mplist = preproc.materials.mplist
    mpplot = preproc.materials.mpplot
    mpread = preproc.materials.mpread
    mptemp = preproc.materials.mptemp
    mptgen = preproc.materials.mptgen
    mptres = preproc.materials.mptres
    mpwrite = preproc.materials.mpwrite
    tbft = preproc.materials.tbft
    uimp = preproc.materials.uimp
    tb = preproc.material_data_tables.tb
    tbcopy = preproc.material_data_tables.tbcopy
    tbdata = preproc.material_data_tables.tbdata
    tbdele = preproc.material_data_tables.tbdele
    tbeo = preproc.material_data_tables.tbeo
    tbfield = preproc.material_data_tables.tbfield
    tbin = preproc.material_data_tables.tbin
    tblist = preproc.material_data_tables.tblist
    tbmodif = preproc.material_data_tables.tbmodif
    tbplot = preproc.material_data_tables.tbplot
    tbpt = preproc.material_data_tables.tbpt
    tbtemp = preproc.material_data_tables.tbtemp
    accat = preproc.meshing.accat
    aclear = preproc.meshing.aclear
    aesize = preproc.meshing.aesize
    amap = preproc.meshing.amap
    amesh = preproc.meshing.amesh
    arefine = preproc.meshing.arefine
    chkmsh = preproc.meshing.chkmsh
    clrmshln = preproc.meshing.clrmshln
    cpcyc = preproc.meshing.cpcyc
    czdel = preproc.meshing.czdel
    czmesh = preproc.meshing.czmesh
    desize = preproc.meshing.desize
    eorient = preproc.meshing.eorient
    erefine = preproc.meshing.erefine
    esize = preproc.meshing.esize
    esys = preproc.meshing.esys
    fvmesh = preproc.meshing.fvmesh
    gsgdata = preproc.meshing.gsgdata
    imesh = preproc.meshing.imesh
    katt = preproc.meshing.katt
    kclear = preproc.meshing.kclear
    kesize = preproc.meshing.kesize
    kmesh = preproc.meshing.kmesh
    krefine = preproc.meshing.krefine
    kscon = preproc.meshing.kscon
    latt = preproc.meshing.latt
    lccat = preproc.meshing.lccat
    lclear = preproc.meshing.lclear
    lesize = preproc.meshing.lesize
    lmesh = preproc.meshing.lmesh
    lrefine = preproc.meshing.lrefine
    mat = preproc.meshing.mat
    mcheck = preproc.meshing.mcheck
    modmsh = preproc.meshing.modmsh
    mopt = preproc.meshing.mopt
    mshape = preproc.meshing.mshape
    mshcopy = preproc.meshing.mshcopy
    mshkey = preproc.meshing.mshkey
    mshmid = preproc.meshing.mshmid
    mshpattern = preproc.meshing.mshpattern
    nrefine = preproc.meshing.nrefine
    psmesh = preproc.meshing.psmesh
    real = preproc.meshing.real
    rthick = preproc.meshing.rthick
    shpp = preproc.meshing.shpp
    smrtsize = preproc.meshing.smrtsize
    tchg = preproc.meshing.tchg
    timp = preproc.meshing.timp
    type = preproc.meshing.type
    vatt = preproc.meshing.vatt
    vclear = preproc.meshing.vclear
    vimp = preproc.meshing.vimp
    vmesh = preproc.meshing.vmesh
    veorient = preproc.meshing.veorient
    vsweep = preproc.meshing.vsweep
    ce = preproc.constraint_equations.ce
    cecyc = preproc.constraint_equations.cecyc
    cedele = preproc.constraint_equations.cedele
    ceintf = preproc.constraint_equations.ceintf
    celist = preproc.constraint_equations.celist
    cerig = preproc.constraint_equations.cerig
    cesgen = preproc.constraint_equations.cesgen
    rbe3 = preproc.constraint_equations.rbe3
    a = preproc.areas.a
    aatt = preproc.areas.aatt
    adele = preproc.areas.adele
    adgl = preproc.areas.adgl
    adrag = preproc.areas.adrag
    afillt = preproc.areas.afillt
    agen = preproc.areas.agen
    al = preproc.areas.al
    alist = preproc.areas.alist
    anorm = preproc.areas.anorm
    aoffst = preproc.areas.aoffst
    aplot = preproc.areas.aplot
    areverse = preproc.areas.areverse
    arotat = preproc.areas.arotat
    arscale = preproc.areas.arscale
    arsym = preproc.areas.arsym
    askin = preproc.areas.askin
    asub = preproc.areas.asub
    asum = preproc.areas.asum
    atran = preproc.areas.atran
    gsum = preproc.areas.gsum
    splot = preproc.areas.splot
    cp = preproc.coupled_dof.cp
    cpdele = preproc.coupled_dof.cpdele
    cpintf = preproc.coupled_dof.cpintf
    cplgen = preproc.coupled_dof.cplgen
    cplist = preproc.coupled_dof.cplist
    cpmerge = preproc.coupled_dof.cpmerge
    cpngen = preproc.coupled_dof.cpngen
    cpsgen = preproc.coupled_dof.cpsgen
    pmlopt = preproc.artificially_matched_layers.pmlopt
    pmlsize = preproc.artificially_matched_layers.pmlsize
    aerocoeff = preproc.special_purpose.aerocoeff
    cint = preproc.special_purpose.cint
    cycfreq = preproc.special_purpose.cycfreq
    cyclic = preproc.special_purpose.cyclic
    cycopt = preproc.special_purpose.cycopt
    emsym = preproc.special_purpose.emsym
    mstole = preproc.special_purpose.mstole
    perbc2d = preproc.special_purpose.perbc2d
    physics = preproc.special_purpose.physics
    race = preproc.special_purpose.race
    sstate = preproc.special_purpose.sstate
    xfdata = preproc.special_purpose.xfdata
    xfenrich = preproc.special_purpose.xfenrich
    xflist = preproc.special_purpose.xflist
    edasmp = preproc.explicit_dynamics.edasmp
    edbound = preproc.explicit_dynamics.edbound
    edbx = preproc.explicit_dynamics.edbx
    edcgen = preproc.explicit_dynamics.edcgen
    edclist = preproc.explicit_dynamics.edclist
    edcmore = preproc.explicit_dynamics.edcmore
    edcnstr = preproc.explicit_dynamics.edcnstr
    edcontact = preproc.explicit_dynamics.edcontact
    edcrb = preproc.explicit_dynamics.edcrb
    edcurve = preproc.explicit_dynamics.edcurve
    eddbl = preproc.explicit_dynamics.eddbl
    eddc = preproc.explicit_dynamics.eddc
    edipart = preproc.explicit_dynamics.edipart
    edlcs = preproc.explicit_dynamics.edlcs
    edmp = preproc.explicit_dynamics.edmp
    ednb = preproc.explicit_dynamics.ednb
    edndtsd = preproc.explicit_dynamics.edndtsd
    ednrot = preproc.explicit_dynamics.ednrot
    edpart = preproc.explicit_dynamics.edpart
    edpc = preproc.explicit_dynamics.edpc
    edsp = preproc.explicit_dynamics.edsp
    edweld = preproc.explicit_dynamics.edweld
    edadapt = preproc.explicit_dynamics.edadapt
    edale = preproc.explicit_dynamics.edale
    edbvis = preproc.explicit_dynamics.edbvis
    edcadapt = preproc.explicit_dynamics.edcadapt
    edcpu = preproc.explicit_dynamics.edcpu
    edcsc = preproc.explicit_dynamics.edcsc
    edcts = preproc.explicit_dynamics.edcts
    eddamp = preproc.explicit_dynamics.eddamp
    eddrelax = preproc.explicit_dynamics.eddrelax
    eddump = preproc.explicit_dynamics.eddump
    edenergy = preproc.explicit_dynamics.edenergy
    edfplot = preproc.explicit_dynamics.edfplot
    edgcale = preproc.explicit_dynamics.edgcale
    edhgls = preproc.explicit_dynamics.edhgls
    edhist = preproc.explicit_dynamics.edhist
    edhtime = preproc.explicit_dynamics.edhtime
    edint = preproc.explicit_dynamics.edint
    edis = preproc.explicit_dynamics.edis
    edload = preproc.explicit_dynamics.edload
    edopt = preproc.explicit_dynamics.edopt
    edout = preproc.explicit_dynamics.edout
    edpl = preproc.explicit_dynamics.edpl
    edpvel = preproc.explicit_dynamics.edpvel
    edrc = preproc.explicit_dynamics.edrc
    edrd = preproc.explicit_dynamics.edrd
    edri = preproc.explicit_dynamics.edri
    edrst = preproc.explicit_dynamics.edrst
    edrun = preproc.explicit_dynamics.edrun
    edshell = preproc.explicit_dynamics.edshell
    edsolv = preproc.explicit_dynamics.edsolv
    edstart = preproc.explicit_dynamics.edstart
    edterm = preproc.explicit_dynamics.edterm
    edtp = preproc.explicit_dynamics.edtp
    edvel = preproc.explicit_dynamics.edvel
    edwrite = preproc.explicit_dynamics.edwrite
    rexport = preproc.explicit_dynamics.rexport
    bsplin = preproc.lines.bsplin
    circle = preproc.lines.circle
    l = preproc.lines.l
    l2ang = preproc.lines.l2ang
    l2tan = preproc.lines.l2tan
    lang = preproc.lines.lang
    larc = preproc.lines.larc
    larea = preproc.lines.larea
    lcomb = preproc.lines.lcomb
    ldele = preproc.lines.ldele
    ldiv = preproc.lines.ldiv
    ldrag = preproc.lines.ldrag
    lextnd = preproc.lines.lextnd
    lfillt = preproc.lines.lfillt
    lgen = preproc.lines.lgen
    llist = preproc.lines.llist
    lplot = preproc.lines.lplot
    lreverse = preproc.lines.lreverse
    lrotat = preproc.lines.lrotat
    lsscale = preproc.lines.lsscale
    lstr = preproc.lines.lstr
    lsum = preproc.lines.lsum
    lsymm = preproc.lines.lsymm
    ltan = preproc.lines.ltan
    ltran = preproc.lines.ltran
    spline = preproc.lines.spline
    ssln = preproc.lines.ssln
    morph = preproc.morphing.morph
    damorph = preproc.morphing.damorph
    demorph = preproc.morphing.demorph
    dvmorph = preproc.morphing.dvmorph
    dig = preproc.digitizing.dig
    dmove = preproc.digitizing.dmove
    dset = preproc.digitizing.dset
    dsurf = preproc.digitizing.dsurf
    blc4 = preproc.primitives.blc4
    blc5 = preproc.primitives.blc5
    block = preproc.primitives.block
    con4 = preproc.primitives.con4
    cone = preproc.primitives.cone
    cyl4 = preproc.primitives.cyl4
    cyl5 = preproc.primitives.cyl5
    cylind = preproc.primitives.cylind
    pcirc = preproc.primitives.pcirc
    poly = preproc.primitives.poly
    pri2 = preproc.primitives.pri2
    prism = preproc.primitives.prism
    ptxy = preproc.primitives.ptxy
    rectng = preproc.primitives.rectng
    rpoly = preproc.primitives.rpoly
    rpr4 = preproc.primitives.rpr4
    rprism = preproc.primitives.rprism
    sph4 = preproc.primitives.sph4
    sph5 = preproc.primitives.sph5
    sphere = preproc.primitives.sphere
    torus = preproc.primitives.torus
    bsax = preproc.sections.bsax
    bsm1 = preproc.sections.bsm1
    bsm2 = preproc.sections.bsm2
    bsmd = preproc.sections.bsmd
    bss1 = preproc.sections.bss1
    bss2 = preproc.sections.bss2
    bste = preproc.sections.bste
    bstq = preproc.sections.bstq
    cbmd = preproc.sections.cbmd
    cbmx = preproc.sections.cbmx
    cbte = preproc.sections.cbte
    cbtmp = preproc.sections.cbtmp
    sdelete = preproc.sections.sdelete
    secmodif = preproc.sections.secmodif
    secfunction = preproc.sections.secfunction
    seccontrol = preproc.sections.seccontrol
    secdata = preproc.sections.secdata
    secjoint = preproc.sections.secjoint
    seclib = preproc.sections.seclib
    seclock = preproc.sections.seclock
    secnum = preproc.sections.secnum
    secoffset = preproc.sections.secoffset
    secplot = preproc.sections.secplot
    secread = preproc.sections.secread
    secstop = preproc.sections.secstop
    sectype = preproc.sections.sectype
    secwrite = preproc.sections.secwrite
    sflex = preproc.sections.sflex
    slist = preproc.sections.slist
    sload = preproc.sections.sload
    ssbt = preproc.sections.ssbt
    ssmt = preproc.sections.ssmt
    sspa = preproc.sections.sspa
    sspb = preproc.sections.sspb
    sspd = preproc.sections.sspd
    sspe = preproc.sections.sspe
    sspm = preproc.sections.sspm
    extopt = preproc.volumes.extopt
    v = preproc.volumes.v
    va = preproc.volumes.va
    vdele = preproc.volumes.vdele
    vdgl = preproc.volumes.vdgl
    vdrag = preproc.volumes.vdrag
    vext = preproc.volumes.vext
    vgen = preproc.volumes.vgen
    vlist = preproc.volumes.vlist
    vlscale = preproc.volumes.vlscale
    voffst = preproc.volumes.voffst
    vplot = preproc.volumes.vplot
    vrotat = preproc.volumes.vrotat
    vsum = preproc.volumes.vsum
    vsymm = preproc.volumes.vsymm
    vtran = preproc.volumes.vtran
    areas = preproc.status.areas
    bool = preproc.status.bool
    ceqn = preproc.status.ceqn
    couple = preproc.status.couple
    digit = preproc.status.digit
    elem = preproc.status.elem
    etype = preproc.status.etype
    febody = preproc.status.febody
    fecons = preproc.status.fecons
    fefor = preproc.status.fefor
    fesurf = preproc.status.fesurf
    geometry = preproc.status.geometry
    keypts = preproc.status.keypts
    line = preproc.status.line
    mater = preproc.status.mater
    meshing = preproc.status.meshing
    nodes = preproc.status.nodes
    prim = preproc.status.prim
    rcon = preproc.status.rcon
    selm = preproc.status.selm
    tble = preproc.status.tble
    volumes = preproc.status.volumes
    k = preproc.keypoints.k
    kbetw = preproc.keypoints.kbetw
    kcenter = preproc.keypoints.kcenter
    kdele = preproc.keypoints.kdele
    kdist = preproc.keypoints.kdist
    kfill = preproc.keypoints.kfill
    kgen = preproc.keypoints.kgen
    kl = preproc.keypoints.kl
    klist = preproc.keypoints.klist
    kmodif = preproc.keypoints.kmodif
    kmove = preproc.keypoints.kmove
    knode = preproc.keypoints.knode
    kplot = preproc.keypoints.kplot
    kpscale = preproc.keypoints.kpscale
    kscale = preproc.keypoints.kscale
    ksum = preproc.keypoints.ksum
    ksymm = preproc.keypoints.ksymm
    ktran = preproc.keypoints.ktran
    source = preproc.keypoints.source
    aadd = preproc.booleans.aadd
    aglue = preproc.booleans.aglue
    aina = preproc.booleans.aina
    ainp = preproc.booleans.ainp
    ainv = preproc.booleans.ainv
    aovlap = preproc.booleans.aovlap
    aptn = preproc.booleans.aptn
    asba = preproc.booleans.asba
    asbl = preproc.booleans.asbl
    asbv = preproc.booleans.asbv
    asbw = preproc.booleans.asbw
    boptn = preproc.booleans.boptn
    btol = preproc.booleans.btol
    lcsl = preproc.booleans.lcsl
    lglue = preproc.booleans.lglue
    lina = preproc.booleans.lina
    linl = preproc.booleans.linl
    linp = preproc.booleans.linp
    linv = preproc.booleans.linv
    lovlap = preproc.booleans.lovlap
    lptn = preproc.booleans.lptn
    lsba = preproc.booleans.lsba
    lsbl = preproc.booleans.lsbl
    lsbv = preproc.booleans.lsbv
    lsbw = preproc.booleans.lsbw
    vadd = preproc.booleans.vadd
    vglue = preproc.booleans.vglue
    vinp = preproc.booleans.vinp
    vinv = preproc.booleans.vinv
    vovlap = preproc.booleans.vovlap
    vptn = preproc.booleans.vptn
    vsba = preproc.booleans.vsba
    vsbv = preproc.booleans.vsbv
    vsbw = preproc.booleans.vsbw
    hptcreate = preproc.hard_points.hptcreate
    hptdelete = preproc.hard_points.hptdelete
    se = preproc.superelements.se
    sedlist = preproc.superelements.sedlist
    selist = preproc.superelements.selist
    sesymm = preproc.superelements.sesymm
    setran = preproc.superelements.setran
    afsurf = preproc.elements.afsurf
    e = preproc.elements.e
    ecpchg = preproc.elements.ecpchg
    edele = preproc.elements.edele
    eextrude = preproc.elements.eextrude
    egen = preproc.elements.egen
    einfin = preproc.elements.einfin
    eintf = preproc.elements.eintf
    elist = preproc.elements.elist
    emid = preproc.elements.emid
    emodif = preproc.elements.emodif
    emore = preproc.elements.emore
    emtgen = preproc.elements.emtgen
    en = preproc.elements.en
    endrelease = preproc.elements.endrelease
    engen = preproc.elements.engen
    enorm = preproc.elements.enorm
    ensym = preproc.elements.ensym
    eplot = preproc.elements.eplot
    eread = preproc.elements.eread
    ereinf = preproc.elements.ereinf
    errang = preproc.elements.errang
    esurf = preproc.elements.esurf
    esym = preproc.elements.esym
    ewrite = preproc.elements.ewrite
    gcdef = preproc.elements.gcdef
    gcgen = preproc.elements.gcgen
    inistate = preproc.elements.inistate
    laylist = preproc.elements.laylist
    layplot = preproc.elements.layplot
    lfsurf = preproc.elements.lfsurf
    ndsurf = preproc.elements.ndsurf
    shsd = preproc.elements.shsd
    swadd = preproc.elements.swadd
    swdel = preproc.elements.swdel
    swgen = preproc.elements.swgen
    swlist = preproc.elements.swlist
    tshap = preproc.elements.tshap
    upgeom = preproc.elements.upgeom
    usrdof = preproc.elements.usrdof
    usrelem = preproc.elements.usrelem
    wtbcreate = preproc.elements.wtbcreate
    center = preproc.nodes.center
    fill = preproc.nodes.fill
    move = preproc.nodes.move
    n = preproc.nodes.n
    naxis = preproc.nodes.naxis
    nang = preproc.nodes.nang
    ndele = preproc.nodes.ndele
    ndist = preproc.nodes.ndist
    ngen = preproc.nodes.ngen
    nkpt = preproc.nodes.nkpt
    nlist = preproc.nodes.nlist
    nmodif = preproc.nodes.nmodif
    nora = preproc.nodes.nora
    norl = preproc.nodes.norl
    nplot = preproc.nodes.nplot
    nread = preproc.nodes.nread
    nrotat = preproc.nodes.nrotat
    nrrang = preproc.nodes.nrrang
    nscale = preproc.nodes.nscale
    nsmooth = preproc.nodes.nsmooth
    nsym = preproc.nodes.nsym
    nwrite = preproc.nodes.nwrite
    quad = preproc.nodes.quad
    transfer = preproc.nodes.transfer


    # APDL commands
    cfclos = apdl.macro_files.cfclos
    cfopen = apdl.macro_files.cfopen
    cfwrite = apdl.macro_files.cfwrite
    create = apdl.macro_files.create
    dflab = apdl.macro_files.dflab
    end = apdl.macro_files.end
    mkdir = apdl.macro_files.mkdir
    msg = apdl.macro_files.msg
    pmacro = apdl.macro_files.pmacro
    psearch = apdl.macro_files.psearch
    rmdir = apdl.macro_files.rmdir
    tee = apdl.macro_files.tee
    ulib = apdl.macro_files.ulib
    use = apdl.macro_files.use
    mfouri = apdl.array_parm.mfouri
    mfun = apdl.array_parm.mfun
    moper = apdl.array_parm.moper
    mwrite = apdl.array_parm.mwrite
    sread = apdl.array_parm.sread
    toper = apdl.array_parm.toper
    vabs = apdl.array_parm.vabs
    vcol = apdl.array_parm.vcol
    vcum = apdl.array_parm.vcum
    vfact = apdl.array_parm.vfact
    vfun = apdl.array_parm.vfun
    vitrp = apdl.array_parm.vitrp
    vlen = apdl.array_parm.vlen
    vmask = apdl.array_parm.vmask
    voper = apdl.array_parm.voper
    starvput = apdl.array_parm.starvput
    vscfun = apdl.array_parm.vscfun
    vstat = apdl.array_parm.vstat
    vwrite = apdl.array_parm.vwrite
    afun = apdl.parameter_definition.afun
    dim = apdl.parameter_definition.dim
    get = apdl.parameter_definition.get
    inquire = apdl.parameter_definition.inquire
    parres = apdl.parameter_definition.parres
    parsav = apdl.parameter_definition.parsav
    starset = apdl.parameter_definition.starset
    taxis = apdl.parameter_definition.taxis
    tread = apdl.parameter_definition.tread
    vfill = apdl.parameter_definition.vfill
    starvget = apdl.parameter_definition.starvget
    vread = apdl.parameter_definition.vread
    abbr = apdl.abbreviations.abbr
    abbres = apdl.abbreviations.abbres
    abbsav = apdl.abbreviations.abbsav
    ucmd = apdl.abbreviations.ucmd
    axpy = apdl.matrix_op.axpy
    comp = apdl.matrix_op.comp
    dmat = apdl.matrix_op.dmat
    dot = apdl.matrix_op.dot
    eigen = apdl.matrix_op.eigen
    export = apdl.matrix_op.export
    fft = apdl.matrix_op.fft
    free = apdl.matrix_op.free
    init = apdl.matrix_op.init
    itengine = apdl.matrix_op.itengine
    lsbac = apdl.matrix_op.lsbac
    lsdump = apdl.matrix_op.lsdump
    lsengine = apdl.matrix_op.lsengine
    lsfactor = apdl.matrix_op.lsfactor
    lsrestore = apdl.matrix_op.lsrestore
    merge = apdl.matrix_op.merge
    mult = apdl.matrix_op.mult
    nrm = apdl.matrix_op.nrm
    remove = apdl.matrix_op.remove
    scal = apdl.matrix_op.scal
    smat = apdl.matrix_op.smat
    starprint = apdl.matrix_op.starprint
    sort = apdl.matrix_op.sort
    vec = apdl.matrix_op.vec
    wrk = apdl.matrix_op.wrk
    wait = apdl.process_controls.wait

    # post26
    cvar = post26_.special.cvar
    pmgtran = post26_.special.pmgtran
    rcyc = post26_.special.rcyc
    resp = post26_.special.resp
    rpsd = post26_.special.rpsd
    smooth = post26_.special.smooth
    vput = post26_.special.vput
    vget = post26_.special.vget
    ansol = post26_.setup.ansol
    cisol = post26_.setup.cisol
    data = post26_.setup.data
    edread = post26_.setup.edread
    enersol = post26_.setup.enersol
    esol = post26_.setup.esol
    file = post26_.setup.file
    gapf = post26_.setup.gapf
    gssol = post26_.setup.gssol
    jsol = post26_.setup.jsol
    nsol = post26_.setup.nsol
    nstore = post26_.setup.nstore
    numvar = post26_.setup.numvar
    reset = post26_.setup.reset
    rforce = post26_.setup.rforce
    rgb = post26_.setup.rgb
    solu = post26_.setup.solu
    store = post26_.setup.store
    timerange = post26_.setup.timerange
    vardel = post26_.setup.vardel
    varnam = post26_.setup.varnam
    define = post26_.status.define
    operate = post26_.status.operate
    plotting = post26_.status.plotting
    print = post26_.status.print
    extrem = post26_.listing.extrem
    lines = post26_.listing.lines
    nprint = post26_.listing.nprint
    prcplx = post26_.listing.prcplx
    prtime = post26_.listing.prtime
    prvar = post26_.listing.prvar
    keep = post26_.display.keep
    plcplx = post26_.display.plcplx
    pltime = post26_.display.pltime
    plvar = post26_.display.plvar
    spread = post26_.display.spread
    xvar = post26_.display.xvar
    cfact = post26_.controls.cfact
    force = post26_.controls.force
    layerp26 = post26_.controls.layerp26
    shell = post26_.controls.shell
    tvar = post26_.controls.tvar
    abs = post26_.operations.abs
    add = post26_.operations.add
    atan = post26_.operations.atan
    clog = post26_.operations.clog
    conjug = post26_.operations.conjug
    deriv = post26_.operations.deriv
    exp = post26_.operations.exp
    filldata = post26_.operations.filldata
    imagin = post26_.operations.imagin
    int1 = post26_.operations.int1
    large = post26_.operations.large
    nlog = post26_.operations.nlog
    prod = post26_.operations.prod
    quot = post26_.operations.quot
    realvar = post26_.operations.realvar
    small = post26_.operations.small
    sqrt = post26_.operations.sqrt

    # post1
    curr2d = post1_.magnetics_calc.curr2d
    emagerr = post1_.magnetics_calc.emagerr
    emf = post1_.magnetics_calc.emf
    emft = post1_.magnetics_calc.emft
    fluxv = post1_.magnetics_calc.fluxv
    mmf = post1_.magnetics_calc.mmf
    plf2d = post1_.magnetics_calc.plf2d
    powerh = post1_.magnetics_calc.powerh
    senergy = post1_.magnetics_calc.senergy
    lcabs = post1_.load_case.lcabs
    lcase = post1_.load_case.lcase
    lcdef = post1_.load_case.lcdef
    lcfact = post1_.load_case.lcfact
    lcfile = post1_.load_case.lcfile
    lcoper = post1_.load_case.lcoper
    lcsel = post1_.load_case.lcsel
    lcsum = post1_.load_case.lcsum
    lcwrite = post1_.load_case.lcwrite
    lczero = post1_.load_case.lczero
    rappnd = post1_.load_case.rappnd
    nsort = post1_.results.nsort
    nusort = post1_.results.nusort
    plcint = post1_.results.plcint
    pldisp = post1_.results.pldisp
    plesol = post1_.results.plesol
    plnsol = post1_.results.plnsol
    plorb = post1_.results.plorb
    prenergy = post1_.results.prenergy
    prorb = post1_.results.prorb
    plvect = post1_.results.plvect
    prcint = post1_.results.prcint
    presol = post1_.results.presol
    prjsol = post1_.results.prjsol
    prnld = post1_.results.prnld
    prnsol = post1_.results.prnsol
    prrfor = post1_.results.prrfor
    prrsol = post1_.results.prrsol
    prvect = post1_.results.prvect
    sumtype = post1_.results.sumtype
    bfint = post1_.special.bfint
    cbdof = post1_.special.cbdof
    cmsfile = post1_.special.cmsfile
    cyccalc = post1_.special.cyccalc
    cycfiles = post1_.special.cycfiles
    cycphase = post1_.special.cycphase
    cycspec = post1_.special.cycspec
    exoption = post1_.special.exoption
    expand = post1_.special.expand
    exprofile = post1_.special.exprofile
    exunit = post1_.special.exunit
    fssparm = post1_.special.fssparm
    fsum = post1_.special.fsum
    hfang = post1_.special.hfang
    hfsym = post1_.special.hfsym
    intsrf = post1_.special.intsrf
    kcalc = post1_.special.kcalc
    nforce = post1_.special.nforce
    nldpost = post1_.special.nldpost
    plcamp = post1_.special.plcamp
    plcfreq = post1_.special.plcfreq
    plchist = post1_.special.plchist
    plfar = post1_.special.plfar
    plmc = post1_.special.plmc
    plnear = post1_.special.plnear
    plzz = post1_.special.plzz
    pras = post1_.special.pras
    prcamp = post1_.special.prcamp
    prfar = post1_.special.prfar
    prnear = post1_.special.prnear
    reswrite = post1_.special.reswrite
    rmflvec = post1_.special.rmflvec
    rsplit = post1_.special.rsplit
    rstmac = post1_.special.rstmac
    spoint = post1_.special.spoint
    spmwrite = post1_.special.spmwrite
    append = post1_.setup.append
    desol = post1_.setup.desol
    dnsol = post1_.setup.dnsol
    hrcplx = post1_.setup.hrcplx
    rescombine = post1_.setup.rescombine
    set = post1_.setup.set
    subset = post1_.setup.subset
    pltrac = post1_.trace_points.pltrac
    trpdel = post1_.trace_points.trpdel
    trplis = post1_.trace_points.trplis
    trpoin = post1_.trace_points.trpoin
    padele = post1_.path_operations.padele
    paget = post1_.path_operations.paget
    paput = post1_.path_operations.paput
    paresu = post1_.path_operations.paresu
    pasave = post1_.path_operations.pasave
    path = post1_.path_operations.path
    pcalc = post1_.path_operations.pcalc
    pcross = post1_.path_operations.pcross
    pdef = post1_.path_operations.pdef
    pdot = post1_.path_operations.pdot
    plpagm = post1_.path_operations.plpagm
    plpath = post1_.path_operations.plpath
    plsect = post1_.path_operations.plsect
    pmap = post1_.path_operations.pmap
    ppath = post1_.path_operations.ppath
    prange = post1_.path_operations.prange
    prpath = post1_.path_operations.prpath
    prsect = post1_.path_operations.prsect
    psel = post1_.path_operations.psel
    pvect = post1_.path_operations.pvect
    detab = post1_.element_table.detab
    esort = post1_.element_table.esort
    etable = post1_.element_table.etable
    eusort = post1_.element_table.eusort
    pletab = post1_.element_table.pletab
    plls = post1_.element_table.plls
    pretab = post1_.element_table.pretab
    sabs = post1_.element_table.sabs
    sadd = post1_.element_table.sadd
    sallow = post1_.element_table.sallow
    sexp = post1_.element_table.sexp
    sfact = post1_.element_table.sfact
    sfcalc = post1_.element_table.sfcalc
    smax = post1_.element_table.smax
    smin = post1_.element_table.smin
    smult = post1_.element_table.smult
    ssum = post1_.element_table.ssum
    tallow = post1_.element_table.tallow
    vcross = post1_.element_table.vcross
    vdot = post1_.element_table.vdot
    ancntr = post1_.animation.ancntr
    ancut = post1_.animation.ancut
    andata = post1_.animation.andata
    andscl = post1_.animation.andscl
    ancyc = post1_.animation.ancyc
    andyna = post1_.animation.andyna
    anfile = post1_.animation.anfile
    anflow = post1_.animation.anflow
    anharm = post1_.animation.anharm
    anim = post1_.animation.anim
    anisos = post1_.animation.anisos
    anmode = post1_.animation.anmode
    anmres = post1_.animation.anmres
    antime = post1_.animation.antime
    trtime = post1_.animation.trtime
    calc = post1_.status.calc
    datadef = post1_.status.datadef
    display = post1_.status.display
    lccalc = post1_.status.lccalc
    point = post1_.status.point
    spec = post1_.status.spec
    format = post1_.listing.format
    header = post1_.listing.header
    irlist = post1_.listing.irlist
    page = post1_.listing.page
    prerr = post1_.listing.prerr
    priter = post1_.listing.priter
    sucalc = post1_.surface_operations.sucalc
    sucr = post1_.surface_operations.sucr
    sudel = post1_.surface_operations.sudel
    sueval = post1_.surface_operations.sueval
    suget = post1_.surface_operations.suget
    sumap = post1_.surface_operations.sumap
    supl = post1_.surface_operations.supl
    supr = post1_.surface_operations.supr
    suresu = post1_.surface_operations.suresu
    susave = post1_.surface_operations.susave
    susel = post1_.surface_operations.susel
    suvect = post1_.surface_operations.suvect
    avprin = post1_.controls.avprin
    avres = post1_.controls.avres
    efacet = post1_.controls.efacet
    ernorm = post1_.controls.ernorm
    inres = post1_.controls.inres
    layer = post1_.controls.layer
    rsys = post1_.controls.rsys
    fc = post1_.failure_criteria.fc
    fccheck = post1_.failure_criteria.fccheck
    fcdele = post1_.failure_criteria.fcdele
    fclist = post1_.failure_criteria.fclist
    fctyp = post1_.failure_criteria.fctyp

    # solution
    gp = solution.gap_conditions.gp
    gpdele = solution.gap_conditions.gpdele
    gplist = solution.gap_conditions.gplist
    f = solution.fe_forces.f
    fcum = solution.fe_forces.fcum
    fdele = solution.fe_forces.fdele
    fj = solution.fe_forces.fj
    fjdele = solution.fe_forces.fjdele
    fjlist = solution.fe_forces.fjlist
    flist = solution.fe_forces.flist
    fscale = solution.fe_forces.fscale
    fssect = solution.fe_forces.fssect
    map2dto3d = solution.twod_to_3d_analysis.map2dto3d
    mfconv = solution.multi_field_solver_convergence_controls.mfconv
    mfiter = solution.multi_field_solver_convergence_controls.mfiter
    mfrelax = solution.multi_field_solver_convergence_controls.mfrelax
    m = solution.master_dof.m
    mdele = solution.master_dof.mdele
    mgen = solution.master_dof.mgen
    mlist = solution.master_dof.mlist
    bf = solution.fe_body_loads.bf
    bfcum = solution.fe_body_loads.bfcum
    bfdele = solution.fe_body_loads.bfdele
    bfe = solution.fe_body_loads.bfe
    bfecum = solution.fe_body_loads.bfecum
    bfedele = solution.fe_body_loads.bfedele
    bfelist = solution.fe_body_loads.bfelist
    bfescal = solution.fe_body_loads.bfescal
    bflist = solution.fe_body_loads.bflist
    bfscale = solution.fe_body_loads.bfscale
    bfunif = solution.fe_body_loads.bfunif
    ldread = solution.fe_body_loads.ldread
    rimport = solution.fe_body_loads.rimport
    tunif = solution.fe_body_loads.tunif
    abextract = solution.analysis_options.abextract
    accoption = solution.analysis_options.accoption
    adams = solution.analysis_options.adams
    antype = solution.analysis_options.antype
    ascres = solution.analysis_options.ascres
    asol = solution.analysis_options.asol
    bcsoption = solution.analysis_options.bcsoption
    cgrow = solution.analysis_options.cgrow
    cmatrix = solution.analysis_options.cmatrix
    cmsopt = solution.analysis_options.cmsopt
    cncheck = solution.analysis_options.cncheck
    cnkmod = solution.analysis_options.cnkmod
    cntr = solution.analysis_options.cntr
    cutcontrol = solution.analysis_options.cutcontrol
    ddoption = solution.analysis_options.ddoption
    dmpext = solution.analysis_options.dmpext
    dmpoption = solution.analysis_options.dmpoption
    dspoption = solution.analysis_options.dspoption
    exbopt = solution.analysis_options.exbopt
    ematwrite = solution.analysis_options.ematwrite
    eqslv = solution.analysis_options.eqslv
    eresx = solution.analysis_options.eresx
    escheck = solution.analysis_options.escheck
    essolv = solution.analysis_options.essolv
    expass = solution.analysis_options.expass
    gauge = solution.analysis_options.gauge
    gmatrix = solution.analysis_options.gmatrix
    lanboption = solution.analysis_options.lanboption
    lumpm = solution.analysis_options.lumpm
    moddir = solution.analysis_options.moddir
    monitor = solution.analysis_options.monitor
    msave = solution.analysis_options.msave
    msolve = solution.analysis_options.msolve
    opncontrol = solution.analysis_options.opncontrol
    outaero = solution.analysis_options.outaero
    ovcheck = solution.analysis_options.ovcheck
    pcgopt = solution.analysis_options.pcgopt
    perturb = solution.analysis_options.perturb
    prscontrol = solution.analysis_options.prscontrol
    pscontrol = solution.analysis_options.pscontrol
    rate = solution.analysis_options.rate
    resvec = solution.analysis_options.resvec
    rstoff = solution.analysis_options.rstoff
    scopt = solution.analysis_options.scopt
    seexp = solution.analysis_options.seexp
    seopt = solution.analysis_options.seopt
    snoption = solution.analysis_options.snoption
    solve = solution.analysis_options.solve
    stabilize = solution.analysis_options.stabilize
    thexpand = solution.analysis_options.thexpand
    thopt = solution.analysis_options.thopt
    rdec = solution.radiosity.rdec
    rsopt = solution.radiosity.rsopt
    rsurf = solution.radiosity.rsurf
    rsymm = solution.radiosity.rsymm
    qsopt = solution.radiosity.qsopt
    ealive = solution.birth_and_death.ealive
    ekill = solution.birth_and_death.ekill
    estif = solution.birth_and_death.estif
    anpres = solution.miscellaneous_loads.anpres
    aport = solution.miscellaneous_loads.aport
    asifile = solution.miscellaneous_loads.asifile
    awave = solution.miscellaneous_loads.awave
    biot = solution.miscellaneous_loads.biot
    dfswave = solution.miscellaneous_loads.dfswave
    fluread = solution.miscellaneous_loads.fluread
    ic = solution.miscellaneous_loads.ic
    icdele = solution.miscellaneous_loads.icdele
    iclist = solution.miscellaneous_loads.iclist
    icrotate = solution.miscellaneous_loads.icrotate
    mpchg = solution.miscellaneous_loads.mpchg
    mrpm = solution.miscellaneous_loads.mrpm
    outpr = solution.miscellaneous_loads.outpr
    outres = solution.miscellaneous_loads.outres
    rescontrol = solution.miscellaneous_loads.rescontrol
    sbclist = solution.miscellaneous_loads.sbclist
    sbctran = solution.miscellaneous_loads.sbctran
    wsprings = solution.miscellaneous_loads.wsprings
    mfcalc = solution.multi_field_solver_time_controls.mfcalc
    mfdtime = solution.multi_field_solver_time_controls.mfdtime
    mfoutput = solution.multi_field_solver_time_controls.mfoutput
    mfrstart = solution.multi_field_solver_time_controls.mfrstart
    mftime = solution.multi_field_solver_time_controls.mftime
    lsclear = solution.load_step_operations.lsclear
    lsdele = solution.load_step_operations.lsdele
    lsread = solution.load_step_operations.lsread
    lssolve = solution.load_step_operations.lssolve
    lswrite = solution.load_step_operations.lswrite
    alphad = solution.dynamic_options.alphad
    betad = solution.dynamic_options.betad
    dmprat = solution.dynamic_options.dmprat
    dmpstr = solution.dynamic_options.dmpstr
    frqscl = solution.dynamic_options.frqscl
    harfrq = solution.dynamic_options.harfrq
    hrexp = solution.dynamic_options.hrexp
    hrocean = solution.dynamic_options.hrocean
    hropt = solution.dynamic_options.hropt
    hrout = solution.dynamic_options.hrout
    lvscale = solution.dynamic_options.lvscale
    mascale = solution.dynamic_options.mascale
    mdamp = solution.dynamic_options.mdamp
    mdplot = solution.dynamic_options.mdplot
    midtol = solution.dynamic_options.midtol
    modcont = solution.dynamic_options.modcont
    modseloption = solution.dynamic_options.modseloption
    modopt = solution.dynamic_options.modopt
    mxpand = solution.dynamic_options.mxpand
    qrdopt = solution.dynamic_options.qrdopt
    rigid = solution.dynamic_options.rigid
    subopt = solution.dynamic_options.subopt
    timint = solution.dynamic_options.timint
    tintp = solution.dynamic_options.tintp
    trnopt = solution.dynamic_options.trnopt
    da = solution.solid_constraints.da
    dadele = solution.solid_constraints.dadele
    dalist = solution.solid_constraints.dalist
    dk = solution.solid_constraints.dk
    dkdele = solution.solid_constraints.dkdele
    dklist = solution.solid_constraints.dklist
    dl = solution.solid_constraints.dl
    dldele = solution.solid_constraints.dldele
    dllist = solution.solid_constraints.dllist
    dtran = solution.solid_constraints.dtran
    addam = solution.spectrum_options.addam
    coval = solution.spectrum_options.coval
    cqc = solution.spectrum_options.cqc
    ddaspec = solution.spectrum_options.ddaspec
    dsum = solution.spectrum_options.dsum
    freq = solution.spectrum_options.freq
    grp = solution.spectrum_options.grp
    mmass = solution.spectrum_options.mmass
    nrlsum = solution.spectrum_options.nrlsum
    pfact = solution.spectrum_options.pfact
    pivcheck = solution.spectrum_options.pivcheck
    psdcom = solution.spectrum_options.psdcom
    psdfrq = solution.spectrum_options.psdfrq
    psdgraph = solution.spectrum_options.psdgraph
    psdres = solution.spectrum_options.psdres
    psdspl = solution.spectrum_options.psdspl
    psdunit = solution.spectrum_options.psdunit
    psdval = solution.spectrum_options.psdval
    psdwav = solution.spectrum_options.psdwav
    qdval = solution.spectrum_options.qdval
    rock = solution.spectrum_options.rock
    rose = solution.spectrum_options.rose
    rigresp = solution.spectrum_options.rigresp
    sed = solution.spectrum_options.sed
    spdamp = solution.spectrum_options.spdamp
    spfreq = solution.spectrum_options.spfreq
    spgraph = solution.spectrum_options.spgraph
    spopt = solution.spectrum_options.spopt
    spunit = solution.spectrum_options.spunit
    spval = solution.spectrum_options.spval
    srss = solution.spectrum_options.srss
    sv = solution.spectrum_options.sv
    svplot = solution.spectrum_options.svplot
    svtyp = solution.spectrum_options.svtyp
    vddam = solution.spectrum_options.vddam
    mfbucket = solution.multi_field_solver_interface_mapping.mfbucket
    mfci = solution.multi_field_solver_interface_mapping.mfci
    mfmap = solution.multi_field_solver_interface_mapping.mfmap
    mftol = solution.multi_field_solver_interface_mapping.mftol
    sf = solution.fe_surface_loads.sf
    sfbeam = solution.fe_surface_loads.sfbeam
    sfcum = solution.fe_surface_loads.sfcum
    sfdele = solution.fe_surface_loads.sfdele
    sfe = solution.fe_surface_loads.sfe
    sfedele = solution.fe_surface_loads.sfedele
    sfelist = solution.fe_surface_loads.sfelist
    sffun = solution.fe_surface_loads.sffun
    sfgrad = solution.fe_surface_loads.sfgrad
    sflist = solution.fe_surface_loads.sflist
    sfscale = solution.fe_surface_loads.sfscale
    autots = solution.load_step_options.autots
    campbell = solution.load_step_options.campbell
    cecmod = solution.load_step_options.cecmod
    deltim = solution.load_step_options.deltim
    expsol = solution.load_step_options.expsol
    kbc = solution.load_step_options.kbc
    kuse = solution.load_step_options.kuse
    magopt = solution.load_step_options.magopt
    magsolv = solution.load_step_options.magsolv
    mode = solution.load_step_options.mode
    nsubst = solution.load_step_options.nsubst
    numexp = solution.load_step_options.numexp
    time = solution.load_step_options.time
    tref = solution.load_step_options.tref
    tsres = solution.load_step_options.tsres
    upcoord = solution.load_step_options.upcoord
    usrcal = solution.load_step_options.usrcal
    wrfull = solution.load_step_options.wrfull
    acel = solution.inertia.acel
    cgloc = solution.inertia.cgloc
    cgomga = solution.inertia.cgomga
    cmacel = solution.inertia.cmacel
    cmdomega = solution.inertia.cmdomega
    cmomega = solution.inertia.cmomega
    cmrotate = solution.inertia.cmrotate
    coriolis = solution.inertia.coriolis
    dcgomg = solution.inertia.dcgomg
    domega = solution.inertia.domega
    irlf = solution.inertia.irlf
    omega = solution.inertia.omega
    synchro = solution.inertia.synchro
    mfcmmand = solution.multi_field_solver_definition_commands.mfcmmand
    mfelem = solution.multi_field_solver_definition_commands.mfelem
    mfem = solution.multi_field_solver_definition_commands.mfem
    mfexter = solution.multi_field_solver_definition_commands.mfexter
    mffname = solution.multi_field_solver_definition_commands.mffname
    arclen = solution.nonlinear_options.arclen
    arctrm = solution.nonlinear_options.arctrm
    bucopt = solution.nonlinear_options.bucopt
    cnvtol = solution.nonlinear_options.cnvtol
    crplim = solution.nonlinear_options.crplim
    gst = solution.nonlinear_options.gst
    lnsrch = solution.nonlinear_options.lnsrch
    ncnv = solution.nonlinear_options.ncnv
    neqit = solution.nonlinear_options.neqit
    nladaptive = solution.nonlinear_options.nladaptive
    nldiag = solution.nonlinear_options.nldiag
    nlgeom = solution.nonlinear_options.nlgeom
    nlhist = solution.nonlinear_options.nlhist
    nlmesh = solution.nonlinear_options.nlmesh
    nropt = solution.nonlinear_options.nropt
    pred = solution.nonlinear_options.pred
    pstres = solution.nonlinear_options.pstres
    bfa = solution.solid_body_loads.bfa
    bfadele = solution.solid_body_loads.bfadele
    bfalist = solution.solid_body_loads.bfalist
    bfk = solution.solid_body_loads.bfk
    bfkdele = solution.solid_body_loads.bfkdele
    bfklist = solution.solid_body_loads.bfklist
    bfl = solution.solid_body_loads.bfl
    bfldele = solution.solid_body_loads.bfldele
    bfllist = solution.solid_body_loads.bfllist
    bftran = solution.solid_body_loads.bftran
    bfv = solution.solid_body_loads.bfv
    bfvdele = solution.solid_body_loads.bfvdele
    bfvlist = solution.solid_body_loads.bfvlist
    fk = solution.solid_forces.fk
    fkdele = solution.solid_forces.fkdele
    fklist = solution.solid_forces.fklist
    ftran = solution.solid_forces.ftran
    sfa = solution.solid_surface_loads.sfa
    sfadele = solution.solid_surface_loads.sfadele
    sfalist = solution.solid_surface_loads.sfalist
    sfl = solution.solid_surface_loads.sfl
    sfldele = solution.solid_surface_loads.sfldele
    sfllist = solution.solid_surface_loads.sfllist
    sftran = solution.solid_surface_loads.sftran
    d = solution.fe_constraints.d
    dcum = solution.fe_constraints.dcum
    ddele = solution.fe_constraints.ddele
    dflx = solution.fe_constraints.dflx
    dj = solution.fe_constraints.dj
    djdele = solution.fe_constraints.djdele
    djlist = solution.fe_constraints.djlist
    dlist = solution.fe_constraints.dlist
    dscale = solution.fe_constraints.dscale
    dsym = solution.fe_constraints.dsym
    dval = solution.fe_constraints.dval
    gsbdata = solution.fe_constraints.gsbdata
    gslist = solution.fe_constraints.gslist
    rezone = solution.rezoning.rezone
    mapsolve = solution.rezoning.mapsolve
    mapvar = solution.rezoning.mapvar
    remesh = solution.rezoning.remesh
    aremesh = solution.rezoning.aremesh
    mfanalysis = solution.multi_field_solver_global_controls.mfanalysis
    mfclear = solution.multi_field_solver_global_controls.mfclear
    mffr = solution.multi_field_solver_global_controls.mffr
    mfinter = solution.multi_field_solver_global_controls.mfinter
    mflist = solution.multi_field_solver_global_controls.mflist
    mforder = solution.multi_field_solver_global_controls.mforder
    mfpsimul = solution.multi_field_solver_global_controls.mfpsimul
    mfsorder = solution.multi_field_solver_global_controls.mfsorder
    mfwrite = solution.multi_field_solver_global_controls.mfwrite
    ocdata = solution.ocean.ocdata
    ocdelete = solution.ocean.ocdelete
    oclist = solution.ocean.oclist
    ocread = solution.ocean.ocread
    octable = solution.ocean.octable
    octype = solution.ocean.octype
    oczone = solution.ocean.oczone
    atype = solution.solution_status.atype
    bioopt = solution.solution_status.bioopt
    deact = solution.solution_status.deact
    dynopt = solution.solution_status.dynopt
    gap = solution.solution_status.gap
    genopt = solution.solution_status.genopt
    inrtia = solution.solution_status.inrtia
    lsoper = solution.solution_status.lsoper
    master = solution.solution_status.master
    nlopt = solution.solution_status.nlopt
    outopt = solution.solution_status.outopt
    smbody = solution.solution_status.smbody
    smcons = solution.solution_status.smcons
    smfor = solution.solution_status.smfor
    smsurf = solution.solution_status.smsurf
    soluopt = solution.solution_status.soluopt
    sptopt = solution.solution_status.sptopt
    mflcomm = solution.multi_field_solver_load_transfer.mflcomm
    mfsurface = solution.multi_field_solver_load_transfer.mfsurface
    mfvolume = solution.multi_field_solver_load_transfer.mfvolume

    # graphics
    an3d = graphics_.annotation.an3d
    annot = graphics_.annotation.annot
    anum = graphics_.annotation.anum
    lspec = graphics_.annotation.lspec
    lsymbol = graphics_.annotation.lsymbol
    pcircle = graphics_.annotation.pcircle
    pmore = graphics_.annotation.pmore
    polygon = graphics_.annotation.polygon
    pspec = graphics_.annotation.pspec
    pwedge = graphics_.annotation.pwedge
    tlabel = graphics_.annotation.tlabel
    tspec = graphics_.annotation.tspec
    axlab = graphics_.graphs.axlab
    grid = graphics_.graphs.grid
    gropt = graphics_.graphs.gropt
    grtyp = graphics_.graphs.grtyp
    gthk = graphics_.graphs.gthk
    xrange = graphics_.graphs.xrange
    yrange = graphics_.graphs.yrange
    color = graphics_.setup.color
    device = graphics_.setup.device
    dsys = graphics_.setup.dsys
    dv3d = graphics_.setup.dv3d
    gcmd = graphics_.setup.gcmd
    gcolumn = graphics_.setup.gcolumn
    gfile = graphics_.setup.gfile
    gplot = graphics_.setup.gplot
    graphics = graphics_.setup.graphics
    gresume = graphics_.setup.gresume
    gsave = graphics_.setup.gsave
    gtype = graphics_.setup.gtype
    image = graphics_.setup.image
    jpeg = graphics_.setup.jpeg
    mrep = graphics_.setup.mrep
    pcopy = graphics_.setup.pcopy
    pngr = graphics_.setup.pngr
    pscr = graphics_.setup.pscr
    pstatus = graphics_.setup.pstatus
    replot = graphics_.setup.replot
    slashreset = graphics_.setup.slashreset
    seg = graphics_.setup.seg
    show = graphics_.setup.show
    tiff = graphics_.setup.tiff
    window = graphics_.setup.window
    cformat = graphics_.labeling.cformat
    clabel = graphics_.labeling.clabel
    contour = graphics_.labeling.contour
    cval = graphics_.labeling.cval
    gformat = graphics_.labeling.gformat
    hbc = graphics_.labeling.hbc
    number = graphics_.labeling.number
    pbc = graphics_.labeling.pbc
    pbf = graphics_.labeling.pbf
    plopts = graphics_.labeling.plopts
    pnum = graphics_.labeling.pnum
    psf = graphics_.labeling.psf
    psymb = graphics_.labeling.psymb
    triad = graphics_.labeling.triad
    udoc = graphics_.labeling.udoc
    cplane = graphics_.style.cplane
    ctype = graphics_.style.ctype
    edge = graphics_.style.edge
    eshape = graphics_.style.eshape
    facet = graphics_.style.facet
    gline = graphics_.style.gline
    gmarker = graphics_.style.gmarker
    gmface = graphics_.style.gmface
    light = graphics_.style.light
    normal = graphics_.style.normal
    shade = graphics_.style.shade
    trlcy = graphics_.style.trlcy
    iclwid = graphics_.scaling.iclwid
    icscale = graphics_.scaling.icscale
    ratio = graphics_.scaling.ratio
    shrink = graphics_.scaling.shrink
    sscale = graphics_.scaling.sscale
    txtre = graphics_.scaling.txtre
    vscale = graphics_.scaling.vscale
    angle = graphics_.views.angle
    auto = graphics_.views.auto
    dist = graphics_.views.dist
    focus = graphics_.views.focus
    user = graphics_.views.user
    vcone = graphics_.views.vcone
    view = graphics_.views.view
    vup = graphics_.views.vup
    xfrm = graphics_.views.xfrm
    zoom = graphics_.views.zoom

    cmap = display_.setup.cmap
    devdisp = display_.setup.devdisp
    filedisp = display_.setup.filedisp
    nocolor = display_.setup.nocolor
    showdisp = display_.setup.showdisp
    trans = display_.setup.trans

    # connection
    cat5in = conn.cat5in
    catiain = conn.catiain
    parain = conn.parain
    proein = conn.proein
    satin = conn.satin
    ugin = conn.ugin

    # hidden
    _batch = hidden.batch
    _output = hidden.output



def ancntr(self, nfram="", delay="", ncycl="", **kwargs):
    """Produces an animated sequence of a contoured deformed shape.

    APDL Command: ANCNTR

    Parameters
    ----------
    nfram
        Number of frames captures (defaults to 5).

    delay
        Time delay during animation (defaults to 0.1 seconds).

    ncycl
        Number of animation cycles (defaults to 5).  Available in non-UI
        mode only.

    Notes
    -----
    ANCNTR involves an ANSYS macro which produces an animation of a
    contoured deformed shape of the last plot action command.  This command
    operates only on graphic display platforms supporting the /SEG command.
    After executing ANCNTR, you can replay the animated sequence by issuing
    the ANIM command.

    The command functions only in the postprocessor.
    """
    command = f"ANCNTR,{nfram},{delay},{ncycl}"
    return self.run(command, **kwargs)


def ancut(self, nfram="", delay="", ncycl="", qoff="", ktop="", topoff="",
          node1="", node2="", node3="", **kwargs):
    """Produces an animated sequence of Q-slices.

    APDL Command: ANCUT

    Parameters
    ----------
    nfram
        Number of frames captures (defaults to 5).

    delay
        Time delay during animation (defaults to 0.1 seconds).

    ncycl
        Number of animation cycles (defaults to 5).  Available in non-UI
        mode only.

    qoff
        Q-slice working plane increment (defaults to .1 half screens).

    ktop
        Topological effect on or off (YES or NO; default is NO).

    topoff
        Topological offset (default is .1 half screens).

    node1
        Node 1 for start of the Q-slice.

    node2
        Node 2 for direction of the Q-slice.

    node3
        Node 3 for plane of the Q-slice.

    Notes
    -----
    ANCUT involves an ANSYS macro which produces an animation of Q-slices
    of the last plot action command.  This command operates only on graphic
    display platforms supporting the /SEG command.  After executing ANCUT,
    you can replay the animated sequence by issuing the ANIM command.

    The command functions only in the postprocessor.
    """
    command = f"ANCUT,{nfram},{delay},{ncycl},{qoff},{ktop},{topoff},{node1},{node2},{node3}"
    return self.run(command, **kwargs)


def andata(self, delay="", ncycl="", rsltdat="", min_="", max_="", incr="",
           frclst="", autocont="", autocntr="", **kwargs):
    """Displays animated graphics data for nonlinear problems.

    APDL Command: ANDATA

    Parameters
    ----------
    delay
        Time delay during animation (defaults to 0.5 seconds).

    ncycl
        Number of animation cycles (defaults to 5).  Available in non-UI
        mode only.

    rsltdat
        The type of results data to be used for the animation sequence.
        This can be:

        0 - Current load step data (default).

        1 - Range of load step data.

        2 - Range of results data.

    min\_
        The range minimum value.  If left blank or 0, defaults to the first
        data point.

    max\_
        The range maximum value.  If left blank or 0, defaults to the last
        data point.

    incr
        The increment between result data (defaults to 1).

    frclst
        Key to force the last sub step in a selected load step to be
        included in the animation (defaults to 0).

    autocont
        A value of 1 enables automatic scaling of contour values based on
        the overall subset range of values. The default value is 0 (no
        automatic scaling).

    autocntr
        A value of 1 disables automatic centering of displaced plots. The
        default value is 0 (allow automatic centering).

    Notes
    -----
    Use the ANDATA command to create animations for nonlinear problems. The
    command works by displaying an individual graphical image for each
    result data set from the results file. For information about creating
    animations for linear problems, see the ANIM command.

    The command operates only on graphic display platforms supporting the
    /SEG command. It uses a macro to produce an animation based on the last
    plot action command (for example, PLDISP).

    The results file must have more than one set of results.

    The command implicitly issues /DSCALE, 1 for default displacement
    scaling. Large displacements may not give good results.

    This command functions only in the postprocessor.
    """
    command = f"ANDATA,{delay},{ncycl},{rsltdat},{min_},{max_},{incr},{frclst},{autocont},,{autocntr}"
    return self.run(command, **kwargs)


def andscl(self, nfram="", delay="", ncycl="", **kwargs):
    """Produces an animated sequence of a deformed shape.

    APDL Command: ANDSCL

    Parameters
    ----------
    nfram
        Number of frames captured (defaults to 5).

    delay
        Time delay during animation (defaults to 0.1 seconds).

    ncycl
        Number of animation cycles (defaults to 5).  Available in non-UI
        mode only.

    Notes
    -----
    ANDSCL involves an ANSYS macro which produces an animation of
    displacement of the last plot action command (for example, PLDISP).
    This command operates only on graphic display platforms supporting the
    /SEG command.  After executing ANDSCL, you can replay the animated
    sequence by issuing the ANIM command.

    The command functions only in the postprocessor.
    """
    command = f"ANDSCL,{nfram},{delay},{ncycl}"
    return self.run(command, **kwargs)


def ancyc(self, numframes="", kcycl="", delay="", **kwargs):
    """Applies a traveling wave animation to graphics data in a modal cyclic

    APDL Command: ANCYC
    symmetry analysis.

    Parameters
    ----------
    numframes
        The number of plot frames for the animation. Valid values range
        from 5 through 36. The default is 18. A low value (because it
        specifies fewer graphical frames) produces a rougher animation but
        loads faster. A high value produces a smoother animation but
        requires more time to load.

    kcycl
        The animation mode:

        0 - Continuous animation cycle (forward-reverse-forward).

        1 - Discontinuous animation cycle (forward-reset-forward). This option is the
            default.

    delay
        The time delay (in seconds) between animation frames. Valid values
        range from 0.1 through 1.0. The default is 0.1 seconds, which
        produces a seemingly real-time animation. A higher value produces a
        slower animation.

    Notes
    -----
    The ANCYC command is valid in a modal cyclic symmetry analysis only.

    The command animates the cyclic symmetry mode shape plot in the General
    Post Processor (/POST1). When you issue a nodal- or element-results
    plot command (for example, PLNSOL, PLESOL, or PLDISP) and then issue
    the ANCYC command, ANSYS applies a traveling wave animation to the mode
    shape plot.

    Each frame of the animation is created by expanding the cyclic symmetry
    mode shape at increasing phase angles (via the /CYCEXPAND command)
    starting at zero in equal increments over 360°. The phase-angle
    increment is 360 / NUMFRAMES.

    The animation display shows the traveling wave of the result quantity
    being plotted. The traveling wave animation is applicable only to nodal
    diameters (harmonic indices) greater than 0 and less than N / 2 (where
    N is the number of cyclic sectors in the model).

    For more information, see Applying a Traveling Wave Animation to the
    Cyclic Model in the Cyclic Symmetry Analysis Guide.
    """
    command = f"ANCYC,{numframes},{kcycl},{delay}"
    return self.run(command, **kwargs)


def andyna(self, delay="", ncycl="", start="", end="", inc="",
           autocontourkey="", **kwargs):
    """Produces an animated sequence of contour values through substeps.

    APDL Command: ANDYNA

    Parameters
    ----------
    delay
        Time delay during animation (defaults to 0.1 seconds).

    ncycl
        Number of animation cycles (defaults to 5).  Available in non-UI
        mode only.

    start
        Number of the starting substep (defaults to 1).

    end
        Number of the ending substep (defaults to the maximum substep).

    inc
        Increment between substeps (defaults to 1).

    autocontourkey
        Auto-scales contour values, based on the overall subset range of
        values  (defaults to 0, no auto-scaling).

    Notes
    -----
    ANDYNA involves an ANSYS macro which produces an animation of contour
    values through all the substeps of the last plot action command.  This
    command operates only on graphic display platforms supporting the /SEG
    command.  After executing ANDYNA, you can replay the animated sequence
    by issuing the ANIM command.

    The command functions only in the postprocessor.
    """
    command = f"ANDYNA,{delay},{ncycl},{start},{end},{inc},{autocontourkey}"
    return self.run(command, **kwargs)


def anfile(self, lab="", fname="", ext="", **kwargs):
    """Saves or resumes an animation sequence to or from a file.

    APDL Command: /ANFILE

    Parameters
    ----------
    lab
        Label type.

        SAVE - Save the current animation to a file.

        RESUME - Resume an animation from a file.

    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    This command saves an animation to a file from local terminal segments
    or resumes an animation from a file to local terminal segments.  See
    the /SEG command for details on segment storage.  See the ANCNTR macro
    for a convenient method of storing graphics frames in terminal memory
    segments.  This command is device dependent and is valid in any
    processor.
    """
    command = f"/ANFILE,{lab},{fname},{ext}"
    return self.run(command, **kwargs)


def anflow(self, nfram="", delay="", ncycl="", time="", spacing="",
           size="", length="", **kwargs):
    """Produces an animated sequence of particle flow in a flowing fluid or a

    APDL Command: ANFLOW
    charged particle traveling in an electric or magnetic field.

    Parameters
    ----------
    nfram
        Number of frames captured (defaults to 5).

    delay
        Time delay during animation (defaults to 0.1 seconds).

    ncycl
        Number of animation cycles (defaults to 5).  Non-UI mode only.

    time
        Total Trace Time (seconds) (defaults to 0, which is the full flow
        trace).

    spacing
        Particle spacing in seconds (defaults to 0).

    size
        Particle size (defaults to 0, which is a line).

    length
        Particle length fraction (defaults to .1).

    Notes
    -----
    ANFLOW invokes an ANSYS macro which produces an animation of particle
    flow in a flowing fluid or charged particle motion in an electric or
    magnetic field by the last plot action command (i.e., PLTRAC).  This
    command is only operational on graphic display platforms supporting the
    /SEG command.  After executing ANFLOW, you can replay the animated
    sequence by issuing the ANIM command.  This command is functional only
    in the Postprocessor.

    The TIME option lets you set the time interval of forward travel for
    the trace.  The SPACING option is used to define the particle spacing
    in seconds from adjacent particles in the stream line.  The SIZE
    variable sets the radius of the particle.  The LENGTH variable is used
    to define the particle length fraction.  By default, the LENGTH is set
    to .1, which means the particle occupies 10% of the flow region and the
    other 90% is a color-code line.  The SPACING and LENGTH variables only
    make sense when the SIZE variable is nonzero (i.e., the particle is
    bigger than the line).
    """
    command = f"ANFLOW,{nfram},{delay},{ncycl},{time},{spacing},{size},{length}"
    return self.run(command, **kwargs)


def anharm(self, nfram="", delay="", ncycl="", nperiod="", cms_antype="",
           cms_modopt="", **kwargs):
    """Produces an animated sequence of time-harmonic results or complex mode

    APDL Command: ANHARM
    shapes.

    Parameters
    ----------
    nfram
        Number of frame captures per cycle.  Defaults to 12.

    delay
        Time delay (seconds) during animation.  Defaults to 0.1 seconds.

    ncycl
        Number of animation cycles.  Defaults to 5.  Not available in the
        GUI.

    nperiod
        Period number for the second set of frames showing the decay or
        growth of a mode shape. Only applies to complex mode shape
        animation. Defaults to 1, animating the decay or growth on the
        first period. Issue -1 to animate without decay or growth.

    cms_antype
        Analysis type performed in the CMS use pass. No default.

        MODAL - Modal analysis

        HARMIC - Harmonic analysis

    cms_modopt
        Mode extraction method selected in the CMS use pass. No default.

        UNSYM - Unsymmetric matrix

        DAMP - Damped system

        QRDAMP - Damped system using QR algorithm

    Notes
    -----
    ANHARM invokes an ANSYS macro which produces an animated sequence of:

    Time-harmonic results in the case of a harmonic analysis
    (ANTYPE,HARMIC)

    Complex mode shapes in the case of a modal analysis (ANTYPE,MODAL).

    In both cases, the results are those of the last plot action (for
    example, PLNSOL,B,SUM).

    The animation converts the complex solution variables (real and
    imaginary sets) into time varying results over one period. For example,
    if NFRAM = 12, then the frame captures are in increments of 30 degree
    phase angles.

    A second set of NFRAM frames will be generated for damped eigenmodes
    from complex eigensolvers to visualize any exponential decay or growth
    of the oscillations. The second set generated will display frames from
    the period number specified by NPERIOD.

    In a CMS analysis, the ANHARM command can be used after the CMS
    expansion pass or the use pass. To use ANHARM after the expansion pass,
    you must indicate whether a modal analysis or a harmonic analysis was
    performed in the CMS use pass by setting CMS_ANTYPE to either MODAL or
    HARMIC. If the use pass was a modal analysis, you must also set the
    CMS_MODOPT field to indicate the mode extraction method that was used
    (UNSYM, DAMP, or QRDAMP). If CMS_MODOPT = QRDAMP, it is assumed that
    CPXMOD was set to ON in the MODOPT command to request complex
    eigenmodes. If the ANHARM command is used after the use pass, it is not
    necessary to set the CMS_ANTYPE or CMS_MODOPT arguments.

    For more information about complex results postprocessing, see POST1
    and POST26 – Complex Results Postprocessing in the Mechanical APDL
    Theory Reference
    """
    command = f"ANHARM,{nfram},{delay},{ncycl},{nperiod},{cms_antype},{cms_modopt}"
    return self.run(command, **kwargs)


def anim(self, ncycl="", kcycl="", delay="", **kwargs):
    """Displays animated graphics data for linear problems.

    APDL Command: ANIM

    Parameters
    ----------
    ncycl
        Number of cycles associated with the animation (defaults to 5 in
        non-GUI mode only)

    kcycl
        Animation mode:

        0 - Continuous animation cycle (forward-reverse-forward-etc.) (default).

        1 - Discontinuous animation cycle (forward-reset-forward-etc.).

    delay
        Time delay (seconds) between animation frames (defaults to 0.1
        seconds).

    Notes
    -----
    Use the ANIM command to create animations for linear problems only. The
    command uses the currently displayed picture based on one particular
    data set from the results file, and linearly interpolates that data
    into different sets, displaying pictures of each interpolated data set
    in sequence to create animation.  For information about creating
    animations for nonlinear problems, see the ANDATA command.

    This command is device-dependent.

    Do not resize the graphic while animation is in progress; doing so can
    result in distorted plots.

    For more information, see the /SEG command for details about segment
    storage, and the ANCNTR macro for a convenient method of storing
    graphics frames in terminal memory segments.

    This command is valid in any processor.
    """
    command = f"ANIM,{ncycl},{kcycl},{delay}"
    return self.run(command, **kwargs)


def anisos(self, nfram="", delay="", ncycl="", **kwargs):
    """Produces an animated sequence of an isosurface.

    APDL Command: ANISOS

    Parameters
    ----------
    nfram
        Number of frames captures (defaults to 9).

    delay
        Time delay during animation (defaults to 0.1 seconds).

    ncycl
        Number of animation cycles (defaults to 5).  Available in non-UI
        mode only.

    Notes
    -----
    ANISOS involves an ANSYS macro which produces an animation of an
    isosurface  of the last plot action command (for example,
    PLNSOL,S,EQV).  The ANISOS command operates only on graphic display
    platforms supporting the /SEG command.  After executing ANISOS, you can
    replay the animated sequence by issuing the ANIM command.

    This command functions only in the postprocessor.
    """
    command = f"ANISOS,{nfram},{delay},{ncycl}"
    return self.run(command, **kwargs)


def anmode(self, nfram="", delay="", ncycl="", kaccel="", **kwargs):
    """Produces an animated sequence of a mode shape.

    APDL Command: ANMODE

    Parameters
    ----------
    nfram
        Number of frames captures (defaults to 5).

    delay
        Time delay during animation (defaults to 0.1 seconds).

    ncycl
        Number of animation cycles (defaults to 5).  Available in non-UI
        mode only.

    kaccel
        Acceleration type:

        0 - Linear acceleration.

        1 - Sinusoidal acceleration.

    Notes
    -----
    ANMODE involves an ANSYS macro which produces an animation of mode
    shape of the last plot action command (for example, PLDISP).  The
    ANMODE command operates only on graphic display platforms supporting
    the /SEG command.  After executing ANMODE, you can replay the animated
    sequence by issuing the ANIM command.

    This command functions only in the postprocessor.
    """
    command = f"ANMODE,{nfram},{delay},{ncycl},{kaccel}"
    return self.run(command, **kwargs)


def anmres(self, delay="", min_="", max_="", inc="", autocntrky="", freq="",
           ext="", **kwargs):
    """Performs animation of results over multiple results files in an

    APDL Command: ANMRES
    explicit dynamic structural analysis or fluid flow analysis with
    remeshing.

    Parameters
    ----------
    delay
        Time delay during animation (default = 0.5 seconds).

    min\_
        Minimum results file number to animate. Default = 1 (for
        Jobname.RS01).

    max\_
        Maximum results file number to animate. Defaults to the highest
        numbered results file, Jobname.RSnn.

    inc
        Increment between results file numbers. Default = 1.

    autocntrky
        Automatic contour scaling option.

        0 - No auto-scaling (default).

        1 - Auto-scaling on.

    freq
         Results frequency key.

        0 or 1 - Animate every results set in each Jobname.EXT file (default).

        2 - Animate every other results set in each Jobname.EXT file.

        n - Animate every nth results set in each Jobname.EXT file.

    ext
        Extension of result files

        'rfl' - Animate Jobname.rflnn

        'rs' - Animate Jobname.rsnn. Default = 'rs'.

    Notes
    -----
    ANMRES invokes an ANSYS macro that performs animation across multiple
    results files (Jobname.EXT, Jobname.EXT, etc.) produced by an explicit
    dynamic structural analysis or fluid flow analysis with remeshing.
    Multiple results files typically occur when adaptive meshing is used in
    an explicit dynamic structural analysis or fluid flow analysis with
    remeshing. Each results file must have more than one set of results.
    ANMRES cannot be used for multiple results files that are caused by
    file splitting.

    ANMRES animates results from files having the currently specified
    jobname (Jobname.EXT - Jobname.EXT).  To change the current jobname,
    use  the /FILNAME command. The animation is based on the last plot
    command (e.g., PLDISP).

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"ANMRES,{delay},{min_},{max_},{inc},{autocntrky},{freq},{ext}"
    return self.run(command, **kwargs)


def antime(self, nfram="", delay="", ncycl="", autocntrky="", rsltdat="",
           min_="", max_="", **kwargs):
    """Produces a  sequential contour animation over a range of time.

    APDL Command: ANTIME

    Parameters
    ----------
    nfram
        Number of frame captures (defaults to 5).

    delay
        Time delay during animation (defaults to 0.1 seconds).

    ncycl
        Number of animation cycles (defaults to 5).  Available in non-UI
        mode only.

    autocntrky
        Auto-scales contour values, based on the overall subset range of
        values.  The auto-scaling option defaults to 0, no auto-scaling.

    rsltdat
        The results data to be used for the animation sequence.  This can
        be:

        0 - Current load step data (default).

        1 - Range of load step data.

        2 - Range of time data.

    min\_
        The range minimum value.  If left blank defaults to the first data
        point.

    max\_
        The range maximum value.  If left blank defaults to the last data
        point.

    Notes
    -----
    The ANTIME command operates only on graphic display platforms
    supporting the /SEG command. It uses an ANSYS macro to produce an
    animation of contour values for the last plot action command (for
    example, PLDISP).  After executing ANTIME, the ANIM command will replay
    the animated sequence.

    This command functions only in the postprocessor.
    """
    command = f"ANTIME,{nfram},{delay},{ncycl},{autocntrky},{rsltdat},{min_},{max_}"
    return self.run(command, **kwargs)


def trtime(self, time="", spacing="", offset="", size="", length="",
           **kwargs):
    """Defines the options used for the PLTRAC (particle flow or charged

    APDL Command: TRTIME
    particle trace) command.

    Parameters
    ----------
    time
        Total Trace Time (seconds) (defaults to 0, which is the full flow
        trace).

    spacing
        Particle spacing in seconds (defaults to 0).

    offset
        Particle offset in seconds (defaults to 0).  It is used internally
        in the ANFLOW macro to produce an animation of particle flow in a
        flowing fluid or charged particle motion in an electric or magnetic
        field.

    size
        Particle size (defaults to 0, which is a line).

    length
        Particle length fraction (defaults to .1).

    Notes
    -----
    The TRTIME command varies the type of   PLTRAC display produced.
    Particle flow or charged particle traces follow a particle's path in
    the forward and backward direction of travel. The DOF selected
    determines the color of the particle trace.  SPACING defines the
    particle spacing in seconds from adjacent particles in the stream line.
    OFFSET defines the offset in seconds from the spacing set by the
    SPACING argument.

    LENGTH defines the particle length fraction.  The default value (.1),
    means the particle occupies 10% of the flow region, and the other 90%
    is a color-coded line.

    SIZE sets the radius of the particle. Use SPACING, OFFSET and LENGTH
    only when SIZE is nonzero (i.e., the particle is bigger than the line).

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"TRTIME,{time},{spacing},{offset},{size},{length}"
    return self.run(command, **kwargs)



from typing import Optional, Union
from ansys.mapdl.core.mapdl_types import MapdlInt, MapdlFloat


def ealive(self, elem: str = "", **kwargs) -> Optional[str]:
    """Reactivates an element (for the birth and death capability).

    APDL Command: EALIVE

    Parameters
    ----------
    elem
        Element to be reactivated:

        ALL  - Reactivates all selected elements (ESEL).

        Comp - Specifies a component name.

    Notes
    -----
    Reactivates the specified element when the birth and death
    capability is being used. An element can be reactivated only
    after it has been deactivated (EKILL).

    Reactivated elements have a zero strain (or thermal heat
    storage, etc.)  state.

    ANSYS, Inc. recommends using the element
    deactivation/reactivation procedure for analyses involving
    linear elastic materials only. Do not use element
    deactivation/reactivation in analyses involving time-
    dependent materials, such as viscoelasticity, viscoplasticity,
    and creep analysis.

    This command is also valid in PREP7.
    """
    command = f"EALIVE,{elem}"
    return self.run(command, **kwargs)


def ekill(self, elem: Union[str, int] = "",
          **kwargs) -> Optional[str]:
    """Deactivates an element (for the birth and death capability).

    APDL Command: EKILL

    Parameters
    ----------
    elem
        Element to be deactivated. If ALL, deactivate all
        selected elements [ESEL]. A component name may also be
        substituted for ELEM.

    Notes
    -----
    Deactivates the specified element when the birth and death
    capability is being used. A deactivated element remains in
    the model but contributes a near-zero stiffness (or
    conductivity, etc.) value (ESTIF) to the overall matrix. Any
    solution-dependent state variables (such as stress, plastic
    strain, creep strain, etc.) are set to zero. Deactivated
    elements contribute nothing to the overall mass (or
    capacitance, etc.) matrix.

    The element can be reactivated with the EALIVE command.

    ANSYS, Inc. recommends using element deactivation/reactivation
    (EKILL/EALIVE) for linear elastic materials only. For all other
    materials, validate the results carefully before using them.

    This command is also valid in PREP7.
    """
    command = f"EKILL,{elem}"
    return self.run(command, **kwargs)


def estif(self, kmult: MapdlFloat = "", **kwargs) -> Optional[str]:
    """Specifies the matrix multiplier for deactivated elements.

    APDL Command: ESTIF

    Parameters
    ----------
    kmult
        Stiffness matrix multiplier for deactivated elements (defaults to
        1.0E-6).

    Examples
    --------
    >>> mapdl.prep7()
    >>> mapdl.estif(1E-8)
    'DEAD ELEMENT STIFFNESS MULTIPLIER= 0.10000E-07'

    Notes
    -----
    Specifies the stiffness matrix multiplier for elements deactivated with
    the EKILL command (birth and death).

    This command is also valid in PREP7.
    """
    command = f"ESTIF,{kmult}"
    return self.run(command, **kwargs)
